package com.unissoft.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.google.common.collect.Maps;
import com.unissoft.MyConstant;
import com.unissoft.model.BasicsDepot;
import com.unissoft.model.BasicsDepotAL;
import com.unissoft.model.BasicsDepotExtend;
import com.unissoft.model.BasicsDepotPO;
import com.unissoft.result.PageParam;
import com.unissoft.result.ResultView;
import com.unissoft.service.BasicsDepotExtendService;
import com.unissoft.service.BasicsDepotService;
import com.unissoft.service.impl.BasicsDepotServiceRecursion;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author root
 * @since 2020-12-03
 */
@Api(description = "分公司与库控制器")
@RestController
@RequestMapping("/basicsDepot")
public class BasicsDepotController {

    @Autowired
    private BasicsDepotService depotService;
    @Autowired
    private BasicsDepotExtendService depotExtendService;
    @Resource
    private BasicsDepotServiceRecursion recursion;

    @Autowired
    private BasicsDepotServiceRecursion serviceRecursion;

    @ApiOperation(value = "分页条件查询", notes = "提交参数：{\n" +
            "  \"condition\":\"{'parentId':'2','pBelonging':''}\",\n" +
            "  \"pageIndex\": 1,\n" +
            "  \"pageSize\": 10\n" +
            "}")
    @PostMapping("/authority/getPage")
    public ResultView getPage(@RequestBody PageParam pageParam) {
        IPage<BasicsDepot> page = depotService.getPage(pageParam);
        return ResultView.success(page);
    }

    @ApiOperation(value = "根据id查询", notes = "")
    @GetMapping("/authority/getById/{id}")
    public ResultView getById(@PathVariable Integer id) {
        BasicsDepot model = depotService.getById(id);
        //查询子表里面的图片地址与企业简介
        List<BasicsDepotExtend> byDepotId = depotExtendService.getByDepotId(id);
        HashMap<String, Object> map = Maps.newHashMap();
        map.put("depot", model);
        map.put("depotExtend", byDepotId);
        return ResultView.success(map);
    }

    @ApiOperation(value = "新增", notes = "")
    @PostMapping("/authority_button/add")
    public ResultView add(@RequestBody BasicsDepot model) {
        depotService.saveDepot(model);
        return ResultView.success(model);
    }

    @ApiOperation(value = "修改", notes = "")
    @PostMapping("/authority_button/update")
    public ResultView update(@RequestBody BasicsDepot model) {
        depotService.update(model);
        return ResultView.success();
    }

    @ApiOperation(value = "根据id删除", notes = "")
    @DeleteMapping("/authority_button/deleteById")
    public ResultView deleteById(@RequestParam int id) {
        if (id != MyConstant.ONE) {
            depotService.deleteById(id);
        }
        return ResultView.success();
    }

    @ApiOperation(value = "查看该是否已经存在;返回结果大于0证明该信息以存在", notes = "")
    @GetMapping("/authority_button/getByPersonnelName")
    public ResultView getByRoleName(@RequestParam String dName) {
        QueryWrapper<BasicsDepot> ew = new QueryWrapper<>();
        ew.eq("d_name", dName);
        int count = depotService.count(ew);
        return ResultView.success(count);
    }

    @ApiOperation(value = "根据库id查询分公司与库下拉选", notes = "")
    @GetMapping("/authority/getDepotAll")
    public ResultView getDepotAll(@RequestParam(required = false) Integer depotId) {
        ArrayList<HashMap<String, String>> depotAll = depotService.getDepotAll(depotId);
        return ResultView.success(depotAll);
    }

    @ApiOperation(value = "根据库id获取所有树形结构资源，默认是所有")
    @GetMapping("/authority/getAll")
    public ResultView getAll(@RequestParam(required = false) Integer depotId) {
        List<BasicsDepotPO> all = depotService.getAll(depotId);
        return ResultView.success(all);
    }

    @ApiOperation(value = "根据库id获取所有树形结构带仓房资源，默认是所有")
    @GetMapping("/authority/getAllC")
    public ResultView getAllC(@RequestParam(required = false) Integer depotId) {
        List<BasicsDepotPO> all = depotService.getAllC(depotId);
        return ResultView.success(all);
    }

    @ApiOperation(value = "参数说明：depotId库id; 根据库用户homeType类型获取树形结构带仓房资源，homeType(1.分公司领导首页 2.库主任及以上领导首页 3.员工首页); cfIds仓房id多个使用‘,’隔开,列如：(1,2,3);" +
            "详细说明：homeType=1 查询所有，homeType=2和depotId=？ 查询本库下面的所有仓房以及分库信息(homeType=2,depotId参数必填)，homeType=3和depotId=？和cfIds=？查询本库下有那些仓房信息(homeType=3 depotId和cfIds参数必填);type=1代表查询不带仓房的信息")
    @GetMapping("/authority/getByHomeTypeAllC")
    public ResultView getByHomeTypeAllC(@RequestParam(required = false) Integer depotId, @RequestParam Integer homeType, @RequestParam(required = false) String cfIds, @RequestParam(required = false) Integer type) {
        List<BasicsDepotPO> all = depotService.getByHomeTypeAllC(depotId, homeType, cfIds, type);
        return ResultView.success(all);
    }

    @ApiOperation(value = "根据库点id查询下面所有库，不传值查询所有MAP", notes = "")
    @GetMapping("/authority/getAllMAP")
    public ResultView getAllMAP(@RequestParam(required = false) Integer depotId) {
        Map<Integer, String> all = depotService.getAllMAP(depotId);
        return ResultView.success(all);
    }

    @ApiOperation(value = "查询所有直属库MAP", notes = "")
    @GetMapping("/authority/getAllDirectDepot")
    public ResultView getAllDirectDepot() {
        Map<Integer, String> all = depotService.getAllDirectDepot();
        return ResultView.success(all);
    }

    /**
     * 上传文件
     */
    @ApiOperation(value = "根据库点id图片上传", notes = "参数说明：files要上传的file数组，多个描述信息json格式：[\"测试1\",\"测试2\",\"测试3\"]")
    @PostMapping("/upload")
    public ResultView upload(@RequestParam(name = "files") MultipartFile[] files, @RequestParam(name = "id") Integer id, @RequestParam(name = "remarks") String remarks) throws Exception {
        depotService.uploadFiles(files, id, remarks);
        return ResultView.success();
    }

    /**
     * 查询所有粮库经纬度
     * 分公司 返回所有,市级 返回 直属库+分库 或 分库 ,分库返回 分库条件p_belonging=1
     *
     * @param homeType 区分人员(1.分公司领导首页 2.库主任及以上领导首页 3.员工首页)
     * @param depotId  库id
     * @return
     */
    @ApiOperation(value = "查询所有粮库经纬度", notes = "返回所有")
    @GetMapping("/authority/getAllLONAndLAT")
    public ResultView getAllLONAndLAT(Integer homeType, Integer depotId) {
        List<BasicsDepotAL> allLonLat = depotService.getAllLONAndLAT(homeType, depotId);
        return ResultView.success(allLonLat);
    }

    @ApiOperation(value = "查询本单位及所有下级单位depotId", notes = "返回所有depotId")
    @GetMapping("/authority/getItselfJoinDownDepotId/{depotId}")
    public ResultView getItselfJoinDownDepotId(@PathVariable Integer depotId) {
        List<Integer> allLonLat = recursion.getAllDepotChild(depotId);
        return ResultView.success(allLonLat);
    }

    @ApiOperation(value = "获取库点自己和所有子集id集合", notes = "返回所有")
    @GetMapping("/authority/getAllDepotChild")
    public List<Integer> getAllDepotChild(@RequestParam Integer depotId){
        List<Integer> allDepotChild = serviceRecursion.getAllDepotChild(depotId);
        return allDepotChild;
    }
}
