package com.unissoft.controller;


import com.unissoft.model.BasicsEnum;
import com.unissoft.model.BasicsEnumPO;
import com.unissoft.result.ResultView;
import com.unissoft.service.BasicsEnumService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author root
 * @since 2021-01-06
 */
@Api(description = "枚举控制器")
@RestController
@RequestMapping("/basicsEnum")
public class BasicsEnumController {

    @Autowired
    private BasicsEnumService enumService;

    @ApiOperation(value = "获取所有资源树结构")
    @GetMapping("/authority/getEnumAll")
    public ResultView getEnumAll() {
        List<BasicsEnumPO> enumAll = enumService.getEnumAll();
        return ResultView.success(enumAll);
    }

    @ApiOperation(value = "获取所有资源MAP")
    @GetMapping("/authority/getAll")
    public ResultView getAll() {
        HashMap<String, Object> all = enumService.getAll();
        return ResultView.success(all);
    }

    @ApiOperation(value = "根据id查询", notes = "")
    @GetMapping("/authority/getById/{id}")
    public ResultView getById(@PathVariable int id) {
        BasicsEnum basicsEnum = enumService.getById(id);
        return ResultView.success(basicsEnum);
    }

    @ApiOperation(value = "新增", notes = "")
    @PostMapping("/authority_button/add")
    public ResultView add(@RequestBody BasicsEnum model) {
        Date date = new Date();
        model.setCreateDate(date);
        enumService.save(model);
        return ResultView.success(model);
    }

    @ApiOperation(value = "修改", notes = "")
    @PostMapping("/authority_button/update")
    public ResultView update(@RequestBody BasicsEnum model) {
        Date date = new Date();
        model.setCreateDate(date);
        enumService.updateById(model);
        return ResultView.success(model);
    }

    @ApiOperation(value = "根据id删除", notes = "")
    @DeleteMapping("/authority_button/deleteById")
    public ResultView deleteById(@RequestParam int id) {
        enumService.deleteById(id);
        return ResultView.success();
    }

    /**
     * 获取只有一个子集的child数据
     * @param parentId 父id
     * @return
     */
    @ApiOperation(value = "parentId查询试用于一级子集", notes = "int类型parentId")
    @GetMapping("/authority/getOnlyChildData/{parentId}")
    public ResultView getOnlyChildData(@PathVariable("parentId") int parentId) {
        List<BasicsEnum> childData = enumService.getOnlyChildData(parentId);
        return ResultView.success(childData);
    }

}
