package com.unissoft.controller;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.unissoft.model.BasicsDepot;
import com.unissoft.model.BasicsDepotPO;
import com.unissoft.model.BasicsWarehouse;
import com.unissoft.model.BasicsWarehousePO;
import com.unissoft.result.PageParam;
import com.unissoft.result.ResultView;
import com.unissoft.service.BasicsDepotService;
import com.unissoft.service.BasicsWarehouseService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author root
 * @since 2020-12-03
 */
@Api(description = "仓房管理控制器")
@RestController
@RequestMapping("/basicsWarehouse")
public class BasicsWarehouseController {

    @Autowired
    private BasicsWarehouseService warehouseService;

    @ApiOperation(value = "分页条件查询根据库id查询仓房信息", notes = "提交参数：{\n" +
            "  \"condition\":\"{'depotId':'1'}\",\n" +
            "  \"pageIndex\": 1,\n" +
            "  \"pageSize\": 10\n" +
            "}")
    @PostMapping("/authority/getPage")
    public ResultView getPage(@RequestBody PageParam pageParam) {
        Page<BasicsWarehouse> page = warehouseService.getPage(pageParam);
        return ResultView.success(page);
    }

    @ApiOperation(value = "根据id查询", notes = "")
    @GetMapping("/authority/getById/{id}")
    public ResultView getById(@PathVariable int id) {
        BasicsWarehouse model = warehouseService.getById(id);
        return ResultView.success(model);
    }

    @ApiOperation(value = "新增", notes = "")
    @PostMapping("/authority_button/add")
    public ResultView add(@RequestBody BasicsWarehouse model) {
        Date date = new Date();
        model.setCreateTime(date);
        warehouseService.save(model);
        return ResultView.success();
    }

    @ApiOperation(value = "修改", notes = "")
    @PostMapping("/authority_button/update")
    public ResultView update(@RequestBody BasicsWarehouse model) {
        Date date = new Date();
        model.setCreateTime(date);
        warehouseService.updateById(model);
        return ResultView.success();
    }

    @ApiOperation(value = "根据id删除", notes = "")
    @DeleteMapping("/authority_button/deleteById")
    public ResultView deleteById(@RequestParam int id) {
        warehouseService.removeById(id);

        return ResultView.success();
    }

    @ApiOperation(value = "根据库点id和仓房名称查看是否已经存在;返回结果大于0证明该信息以存在", notes = "")
    @GetMapping("/authority_button/getByPersonnelName")
    public ResultView getByRoleName(@RequestParam Integer depotId, @RequestParam String hName) {
        QueryWrapper<BasicsWarehouse> ew = new QueryWrapper<>();
        ew.eq("depot_id", depotId);
        ew.eq("h_name", hName);
        int count = warehouseService.count(ew);
        return ResultView.success(count);
    }

    @ApiOperation(value = "根据库id查询分公司与库下拉选", notes = "")
    @GetMapping("/authority/getDepotAll")
    public ResultView getDepotAll(@RequestParam Integer depotId) {
        ArrayList<HashMap<String, String>> depotAll = warehouseService.getDepotAll(depotId);
        return ResultView.success(depotAll);
    }

    @ApiOperation(value = "根据库点id查询下面所有仓房MAP", notes = "")
    @GetMapping("/authority/getAllMAP")
    public ResultView getAllMAP(@RequestParam(required = false) Integer depotId) {
        Map<Integer, String> all = warehouseService.getAllMAP(depotId);
        return ResultView.success(all);
    }

    @ApiOperation(value = "分公司领导首页省图.市图.库图。储存状态.粮食种类.仓房状态", notes = "")
    @PostMapping("/authority/getPercent")
    public ResultView getHouseStatus(@RequestBody BasicsWarehouse BasicsWarehouse) {
        Map<String, List<BasicsWarehousePO>> model = warehouseService.getPercent(BasicsWarehouse);
        return ResultView.success(model);
    }

    @ApiOperation(value = "首页地图", notes = "")
    @GetMapping("/authority/getPictureAllMApicture")
    public ResultView getPictureAllMApicture(@RequestParam(required = false) Integer depotId) {
        List<BasicsWarehouse> all = warehouseService.getPictureAllMApicture(depotId);
        return ResultView.success(all);
    }

    @ApiOperation(value = "根据用户所拥有的库id和仓房id查询树结构", notes = "")
    @GetMapping("/authority/getByDepotIDAllC")
    public ResultView getByDepotIDAllC(@RequestParam(required = false) Integer depotId, @RequestParam(required = false) String cfIds) {
        List<BasicsDepotPO> depotAll = warehouseService.getByDepotIDAllC(depotId, cfIds);
        return ResultView.success(depotAll);
    }

    @ApiOperation(value = "根据id查询", notes = "")
    @GetMapping("/authority/getByIdHouse")
    public BasicsWarehouse getById(@RequestParam("id") Integer id) {
        BasicsWarehouse model = warehouseService.getById(id);
        return model;
    }
}
