package com.unissoft.controller;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.unissoft.model.ProjectModel;
import com.unissoft.result.PageParam;
import com.unissoft.result.ResultView;
import com.unissoft.service.ProjectService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.Date;
import java.util.Map;

/**
 * <p>
 * 项目（如公司里某个项目） 前端控制器
 * </p>
 *
 * @author wsm
 * @since 2019-07-16
 */
@Api(description = "项目控制器")
@RestController
@RequestMapping("/project")
public class ProjectController {

    @Autowired
    private ProjectService projectService;

    @ApiOperation(value = "分页条件查询", notes = "提交参数：{\"pageIndex\":1,\"pageSize\":10,\"sort\":\"name desc\",\"condition\":\"{\'name\':\'\',\'code\':\'\',\'regionId\':\'\',\'isEnable\':\'\'}\"}")
    @PostMapping("/authority/getPage")
    public ResultView getPage(@RequestBody PageParam pageParam) {
        Page<ProjectModel> page = projectService.getPage(pageParam);
        return ResultView.success(page);
    }

    @ApiOperation(value = "根据id查询", notes = "")
    @GetMapping("/authority/getById/{id}")
    public ResultView getById(@PathVariable Long id) {
        ProjectModel model = projectService.getById(id);
        return ResultView.success(model);
    }

    @ApiOperation(value = "新增", notes = "")
    @PostMapping("/authority_button/add")
    public ResultView add(@RequestBody ProjectModel model) {
        Date date = new Date();
//        model.setId(CreateUtil.id());
        model.setCreateTime(date);
        model.setUpdateTime(date);
        projectService.save(model);
        return ResultView.success(model);
    }

    @ApiOperation(value = "修改", notes = "")
    @PostMapping("/authority_button/update")
    public ResultView update(@RequestBody ProjectModel model) {
        Date date = new Date();
        model.setUpdateTime(date);
        projectService.updateById(model);
        return ResultView.success(model);
    }

    @ApiOperation(value = "根据id删除", notes = "")
    @DeleteMapping("/authority_button/deleteById")
    public ResultView deleteById(@RequestParam Long id) {
        projectService.removeById(id);
        return ResultView.success();
    }

    @ApiOperation(value = "根据ids删除", notes = "")
    @DeleteMapping("/authority_button/deleteByIds")
    public ResultView deleteByIds(@RequestParam Long[] ids) {
        projectService.removeByIds(Arrays.asList(ids));
        return ResultView.success();
    }

    @ApiOperation(value = "查询所有部门信息MAP", notes = "")
    @GetMapping("/authority/getAllMAP")
    public ResultView getAllMAP() {
        Map<Integer, String> all = projectService.getAllMAP();
        return ResultView.success(all);
    }
}

