package com.unissoft.controller;

import com.google.common.collect.Maps;
import com.unissoft.MyConstant;
import com.unissoft.dto.ResourceRoleInfoDto;
import com.unissoft.feign.OauthFeign;
import com.unissoft.result.ResultEnum;
import com.unissoft.result.ResultView;
import com.unissoft.service.SysResourceRoleService;
import com.unissoft.service.SysUserService;
import com.unissoft.service.impl.OauthClientDetailsServicel;
import com.unissoft.utils.RedisUtil;
import com.unissoft.webConfig.service.TokenServiceImpl;
import com.wf.captcha.SpecCaptcha;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.List;
import java.util.UUID;

/**
 * 公共控制器
 */
@Api(description = "公共控制器")
@RestController
@RequestMapping("/public")
public class PublicController {

    @Autowired
    private RedisUtil redisUtil;

    @Autowired
    private SysUserService userService;

    @Autowired
    private OauthClientDetailsServicel oauthClientDetailsService;

    @Autowired
    private OauthFeign oauthFeign;

    @Autowired
    private SysResourceRoleService resourceRoleService;

    @Autowired
    private TokenServiceImpl tokenService;

    @ApiOperation(value = "注册", notes = "提交参数：{\"uName\":\"wsm3\",\"uPassword\":\"123456\"}")
    @PostMapping("/register")
    public ResultView register(@RequestBody String body) {
        userService.register(body);
        return ResultView.success();
    }

    @ApiOperation(value = "登录", notes = "提交参数：{\"uName\":\"wsm1\",\"uPassword\":\"123456\"}")
    @PostMapping("/login")
    public ResultView login(@RequestBody String body) {
        return userService.login(body);
    }

    @ApiOperation(value = "注销登录")
    @GetMapping("/logout")
    public ResultView logout(@RequestParam String token, @RequestParam(required = false) String personnelId) {
        ResultView resultView = userService.logout(token, personnelId);
        return resultView;
    }

    @ApiOperation(value = "查当前项目的资源角色信息")
    @GetMapping("/getResourceRoleInfo")
    public List<ResourceRoleInfoDto> getResourceRoleInfo(@RequestParam String projectCode) {
        return resourceRoleService.getResourceRoleInfo(projectCode);
    }

    @ApiOperation(value = "根据client_id查询客户端resource_ids")
    @GetMapping("/getResourceIdsByClientId")
    public String getResourceIdsByClientId(@RequestParam String clientId) {
        String resourceIds = oauthClientDetailsService.getById(clientId).getResourceIds();
        return resourceIds;
    }

    @ApiOperation(value = "根据地址刷新各服务权限")
    @GetMapping("/loadResourceDefine")
    public ResultView loadResourceDefine() {
        userService.loadResourceDefine();
        return ResultView.success();
    }

    @ApiOperation(value = "获取登录验证码")
    @GetMapping("/captcha")
    public ResultView captcha() {
        SpecCaptcha specCaptcha = new SpecCaptcha(130, 48, 4);
        String verCode = specCaptcha.text().toLowerCase();
        String key = UUID.randomUUID().toString();
        // 存入redis并设置过期时间为3分钟
        redisUtil.set(key, verCode, MyConstant.REDIS_SECONDS);
        // 将key和base64返回给前端
        HashMap<String, String> map = Maps.newHashMap();
        map.put("key", key);
        map.put("image", specCaptcha.toBase64());
        return ResultView.success(map);
    }

    @ApiOperation(value = "登录验证码验证")
    @GetMapping("/verifyCode")
    public ResultView verifyCode(@RequestParam String key, @RequestParam String verCode) {
        // 获取redis中的验证码
        String redisCode = redisUtil.get(key).toString();
        // 判断验证码
        if (verCode == null || !redisCode.equals(verCode.trim().toLowerCase())) {
            return ResultView.error(ResultEnum.CODE_4);
        }
        return ResultView.success(ResultEnum.CODE_12);
    }

    @ApiOperation(value = "获取幂等token", notes = "")
    @GetMapping("/getToken")
    public ResultView getToken() {
        String token = tokenService.createToken();
        return ResultView.success(token);
    }
}
