package com.unissoft.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.unissoft.model.SysResource;
import com.unissoft.model.SysResourcePO;
import com.unissoft.model.SysRole;
import com.unissoft.result.PageParam;
import com.unissoft.result.ResultView;
import com.unissoft.service.SysResourceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 资源（如前台某个菜单） 前端控制器
 * </p>
 *
 * @author wsm
 * @since 2019-07-16
 */
@Api(description = "资源控制器")
@RestController
@RequestMapping("/resource")
public class ResourceController {

    @Autowired
    private SysResourceService resourceService;

    @ApiOperation(value = "获取所有树形结构资源")
    @PostMapping("/authority/getAll")
    public ResultView getAll() {
        List<SysResourcePO> all = resourceService.getAll();
        return ResultView.success(all);
    }

    @ApiOperation(value = "分页条件查询", notes = "提交参数：{\"pageIndex\":1,\"pageSize\":10,\"sort\":\"name desc\",\"condition\":\"{\'name\':\'\',\'code\':\'\',\'projectId\':\'\',\'isEnable\':\'\'}\"}")
    @PostMapping("/authority/getPage")
    public ResultView getPage(@RequestBody PageParam pageParam) {
        Page<SysResource> page = resourceService.getPage(pageParam);
        return ResultView.success(page);
    }

    @ApiOperation(value = "根据id查询", notes = "")
    @GetMapping("/authority/getById/{id}")
    public ResultView getById(@PathVariable int id) {
        SysResource model = resourceService.getById(id);
        return ResultView.success(model);
    }

    @ApiOperation(value = "新增", notes = "")
    @PostMapping("/authority_button/add")
    public ResultView add(@RequestBody SysResource model) {
        Date date = new Date();
        if (model.getSeParentId() == null){
            model.setSeParentId(0);
        }
        model.setSeCreateTime(date);
        model.setSeUpdateTime(date);
        resourceService.save(model);
        return ResultView.success(model);
    }

    @ApiOperation(value = "修改", notes = "")
    @PostMapping("/authority_button/update")
    public ResultView update(@RequestBody SysResource model) {
        Date date = new Date();
        model.setSeUpdateTime(date);
        resourceService.updateById(model);
        return ResultView.success(model);
    }

    @ApiOperation(value = "根据id删除", notes = "")
    @DeleteMapping("/authority_button/deleteById")
    public ResultView deleteById(@RequestParam int id) {
        resourceService.removeById(id);
        return ResultView.success();
    }

    @ApiOperation(value = "查看该url是否已经存在;返回结果大于0证明该url已存在", notes = "")
    @GetMapping("/authority_button/getByResourceUrl")
    public ResultView getByResourceUrl(@RequestParam String url) {
        QueryWrapper<SysResource> ew = new QueryWrapper<>();
        ew.eq("se_button", url.trim());
        int count = resourceService.count(ew);
        return ResultView.success(count);
    }
}

