package com.unissoft.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.unissoft.model.SysRole;
import com.unissoft.result.PageParam;
import com.unissoft.result.ResultView;
import com.unissoft.service.SysRoleService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * <p>
 * 角色 前端控制器
 * </p>
 *
 * @author wsm
 * @since 2019-07-18
 */
@Api(description = "角色控制器")
@RestController
@RequestMapping("/role")
public class RoleController {

    @Autowired
    private SysRoleService roleService;

    @ApiOperation(value = "根据角色id查询角色下有那些资源", notes = "")
    @GetMapping("/authority/getByColeId")
    public ResultView getByColeId(@RequestParam Integer id) {
        List<Integer> byColeId = roleService.getByColeId(id);
        return ResultView.success(byColeId);
    }

    @ApiOperation(value = "根据角色id查询角色下有那些资源带ParentId", notes = "")
    @GetMapping("/authority/getAllResource")
    public ResultView getAllResource(@RequestParam Integer id) {
        List<Integer> allResource = roleService.getAllResource(id);
        return ResultView.success(allResource);
    }

    @ApiOperation(value = "查询所有角色信息", notes = "")
    @GetMapping("/authority/getAll")
    public ResultView getAll() {
        ArrayList<HashMap<String, String>> orleAll = roleService.getAll();
        return ResultView.success(orleAll);
    }

    @ApiOperation(value = "分页条件查询", notes = "提交参数：{\"pageIndex\":1,\"pageSize\":10,\",\"condition\":\"{\'rName\':\'\"}")
    @PostMapping("/authority/getPage")
    public ResultView getPage(@RequestBody PageParam pageParam) {
        Page<SysRole> page = roleService.getPage(pageParam);
        return ResultView.success(page);
    }

    @ApiOperation(value = "根据id查询", notes = "")
    @GetMapping("/authority/getById/{id}")
    public ResultView getById(@PathVariable Long id) {
        SysRole model = roleService.getById(id);
        return ResultView.success(model);
    }

    @ApiOperation(value = "新增", notes = "")
    @PostMapping("/authority_button/add")
    public ResultView add(@RequestBody SysRole model) {
        roleService.saveRole(model);
        return ResultView.success(model);
    }

    @ApiOperation(value = "修改", notes = "")
    @PostMapping("/authority_button/update")
    public ResultView update(@RequestBody SysRole model) {
        roleService.updateRole(model);
        return ResultView.success(model);
    }

    @ApiOperation(value = "根据id删除", notes = "")
    @DeleteMapping("/authority_button/deleteById")
    public ResultView deleteById(@RequestParam Integer id) {
        roleService.deleteById(id);
        return ResultView.success();
    }

    @ApiOperation(value = "查看该角色是否已经存在;返回结果大于0证明该角色以存在", notes = "")
    @GetMapping("/authority_button/getByRoleName")
    public ResultView getByRoleName(@RequestParam String rName) {
        QueryWrapper<SysRole> ew = new QueryWrapper<>();
        ew.eq("r_name", rName);
        int count = roleService.count(ew);
        return ResultView.success(count);
    }

}

