package com.unissoft.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.unissoft.model.SysPersonnel;
import com.unissoft.model.SysPersonnelPO;
import com.unissoft.result.PageParam;
import com.unissoft.result.ResultView;
import com.unissoft.service.SysPersonnelService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.models.auth.In;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.*;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author root
 * @since 2020-12-01
 */
@Api(description = "人员控制器")
@RestController
@RequestMapping("/sysPersonnel")
public class SysPersonnelController {

    @Autowired
    private SysPersonnelService personnelService;

    @ApiOperation(value = "分页条件查询", notes = "提交参数：{\"pageIndex\":1,\"pageSize\":10,\"condition\":\"{\'depotId\':\'\'}\"}")
    @PostMapping("/authority/getPage")
    public ResultView getPage(@RequestBody PageParam pageParam) {
        Page<SysPersonnel> pageResult = personnelService.getPage(pageParam);
        return ResultView.success(pageResult);
    }

    @ApiOperation(value = "根据库点id和部门id查询下面的人员，另加统计等级", notes = "提交参数：{\n" +
            "  \"condition\": \"{'depotId':'1','departmentId':'1'}\",\n" +
            "  \"pageIndex\": 1,\n" +
            "  \"pageSize\": 10\n" +
            "}")
    @PostMapping("/authority/getByDepartmentPage")
    public ResultView getByDepartmentPage(@RequestBody PageParam pageParam) {
        Map<String,Object> pageResult = personnelService.getByDepartmentPage(pageParam);
        return ResultView.success(pageResult);
    }

    @ApiOperation(value = "根据id查询", notes = "")
    @GetMapping("/authority/getById/{id}")
    public ResultView getById(@PathVariable int id) {
        SysPersonnel model = personnelService.getById(id);
        return ResultView.success(model);
    }

    @ApiOperation(value = "根据仓房id查询保管员信息", notes = "")
    @GetMapping("/authority/getByWarehouseId")
    public ResultView getByWarehouseId(@RequestParam int depotId,@RequestParam int warehouseId) {
        SysPersonnel model = personnelService.getByWarehouseId(depotId,warehouseId);
        return ResultView.success(model);
    }

    @ApiOperation(value = "新增", notes = "")
    @PostMapping("/authority_button/add")
    public ResultView add(@RequestBody SysPersonnel model) {
        Date date = new Date();
        model.setCreateTime(date);
        model.setuStatus(1);
        personnelService.save(model);
        return ResultView.success(model);
    }

    @ApiOperation(value = "修改", notes = "")
    @PostMapping("/authority_button/update")
    public ResultView update(@RequestBody SysPersonnel model) {
        Date date = new Date();
        model.setCreateTime(date);
        model.setuStatus(1);
        personnelService.updateById(model);
        return ResultView.success();
    }

    @ApiOperation(value = "根据id删除", notes = "")
    @DeleteMapping("/authority_button/deleteById")
    public ResultView deleteById(@RequestParam int id) {
//        personnelService.removeById(id);
        SysPersonnel model = personnelService.getById(id);
        model.setuStatus(3);
        personnelService.updateById(model);
        return ResultView.success();
    }

    @ApiOperation(value = "查看该人员是否已经存在;返回结果大于0证明该人员以存在", notes = "")
    @GetMapping("/authority_button/getByPersonnelName")
    public ResultView getByRoleName(@RequestParam String uName) {
        QueryWrapper<SysPersonnel> ew = new QueryWrapper<>();
        ew.eq("u_name", uName);
        int count = personnelService.count(ew);
        return ResultView.success(count);
    }

    @ApiOperation(value = "查询所有用户，根据库ID查询人员下拉选", notes = "")
    @GetMapping("/authority/getAll")
    public ResultView getAll(@RequestParam int depotId) {
        ArrayList<HashMap<String, String>> all = personnelService.getAll(depotId);
        return ResultView.success(all);
    }

    @ApiOperation(value = "查询所有用户，根据库ID查询人员下拉选MAP", notes = "")
    @GetMapping("/authority/getDepotIdAll")
    public ResultView getDepotIdAll(@RequestParam int depotId) {
        HashMap<Integer, String> all = personnelService.getDepotIdAll(depotId);
        return ResultView.success(all);
    }

    /**
     * 上传文件
     */
    @ApiOperation(value = "根据用户id图片上传", notes = "")
    @PostMapping("/upload")
    public ResultView upload(@RequestParam(name = "file") MultipartFile file, @RequestParam(name = "id") Integer id) throws Exception {
        String images = personnelService.uploadFile(file, id);
        return ResultView.success(images);
    }

    @ApiOperation(value = "根据库点id和角色id获取用户下拉选", notes = "")
    @GetMapping("/authority_button/getByDepotIdAndRoleId")
    public ResultView getByDepotIdAndRoleId(@RequestParam Integer depotID,@RequestParam Integer roleID) {
        ArrayList<HashMap<String, String>> byDepotIdAndRoleId = personnelService.getByDepotIdAndRoleId(depotID, roleID);
        return ResultView.success(byDepotIdAndRoleId);
    }

    @ApiOperation(value = "根据库点depotId，获取库每个部门下有那些人员树结构", notes = "")
    @GetMapping("/authority/getByDepotId")
    public ResultView getByDepotId(@RequestParam Integer depotId) {
        List<SysPersonnelPO> byDepotId = personnelService.getByDepotId(depotId);
        return ResultView.success(byDepotId);
    }
}
