package com.unissoft.controller;

import com.unissoft.model.SysUserPO;
import com.unissoft.result.ResultView;
import com.unissoft.service.impl.BasicsDepotServiceRecursion;
import com.unissoft.utils.RedisUtil;
import com.unissoft.utils.UserUtils;
import com.unissoft.webConfig.service.impl.AccessLimit;
import com.unissoft.webConfig.service.impl.AutoIdempotent;
import com.unissoft.webConfig.service.TokenServiceImpl;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 测试控制器
 */
@Api(description = "测试类")
@RestController
@RefreshScope
@RequestMapping("/test")
public class TestCotroller {

    @Autowired
    private RedisUtil redisUtil;

    @Autowired
    private TokenServiceImpl tokenService;

    @ApiOperation(value = "redis存入list测试", notes = "")
    @GetMapping("/redisSet")
    @ResponseBody
    public Object set() {

       /* List<Object> strings = new ArrayList<>();
        strings.add("哈哈哈");
        strings.add("嘤嘤嘤");
        strings.add("嘎嘎嘎");

        redisUtil.lSetList("666", strings);*/

        redisUtil.set("aa", "123456", 60);
        return "set success~";
    }

    @ApiOperation(value = "redis获取list测试", notes = "")
    @GetMapping("/redisGet")
    @ResponseBody
    public Object get() {

        System.out.println(redisUtil.get("aa"));

        return "get success";
    }

    @ApiOperation(value = "redis删除测试", notes = "")
    @GetMapping("/redisDel")
    @ResponseBody
    public Object redisDel() {

        redisUtil.del("aa");

        return "get success";
    }


    /**
     * @param redisKey redis取值的key
     * @param verCode  用户传入的验证码
     * @return
     */
    @ApiOperation(value = "验证码测试", notes = "")
    @GetMapping("/getAll")
    public ResultView getAll(@RequestParam String redisKey, @RequestParam String verCode) {
        // 获取redis中的验证码
        String redisCode = redisUtil.get(redisKey).toString();
        // 判断验证码
        if (verCode == null || !redisCode.equals(verCode.trim().toLowerCase())) {
            return ResultView.error("验证码不正确");
        }
        return ResultView.success("redis:" + redisCode);
//        return ResultView.success("testdemo:"+testdemo);
    }

    @Autowired
    BasicsDepotServiceRecursion depotServiceRecursion;

    @ApiOperation(value = "测试获取所有库的子集，测试幂等", notes = "")
    @AutoIdempotent //测试幂等注解
    @GetMapping("/getDepotChild")
    public ResultView getDepotChild(@RequestParam int depotId) {
        List<Integer> allDepotChild = depotServiceRecursion.getAllDepotChild(depotId);
        return ResultView.success(allDepotChild);
    }

    @ApiOperation(value = "测试获取所有库的子集,测试请求次数", notes = "")
    @AccessLimit(seconds = 5,maxCount = 3) //测试请求次数，表示5秒之内不可超过三次
    @GetMapping("/getDepotChild2")
    public ResultView getDepotChild2(@RequestParam int depotId) {
        List<Integer> allDepotChild = depotServiceRecursion.getAllDepotChild(depotId);
        return ResultView.success(allDepotChild);
    }

    @ApiOperation(value = "获取当前用户", notes = "")
    @GetMapping("/getUser")
    public ResultView getUser(@RequestParam int depotId) {
        SysUserPO currUserInfo = UserUtils.getCurrUserInfo();
        return ResultView.success("user:" + currUserInfo);
    }


    public static void main(String[] args) {
       /* List<String> remarks = new ArrayList<>();
        remarks.add("测试1");
        remarks.add("测试2");
        remarks.add("测试3");
        String s = JSONArray.toJSON(remarks).toString();
        System.out.println(s);
        List<String> list = JSONArray.parseArray(s, String.class);
        System.out.println(list);*/
    }
}
