package com.unissoft.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.unissoft.model.SysUser;
import com.unissoft.model.SysUserDTO;
import com.unissoft.model.SysUserPO;
import com.unissoft.result.PageParam;
import com.unissoft.result.ResultView;
import com.unissoft.service.SysUserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.DigestUtils;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 用户 前端控制器
 * </p>
 *
 * @author wsm
 * @since 2019-01-28
 */
@Api(description = "用户控制器")
@RestController
@RequestMapping("/user")
public class UserController {

    @Autowired
    private SysUserService userService;

    @ApiOperation(value = "分页条件查询", notes = "提交参数：{\"pageIndex\":1,\"pageSize\":10,\"condition\":\"{'depotId':'','roleId':'','uName':'','personnelName':''}\"}")
    @PostMapping("/authority/getPage")
    public ResultView getPage(@RequestBody PageParam pageParam) {
        IPage<SysUserPO> page = userService.getPage(pageParam);
        return ResultView.success(page);
    }

    @ApiOperation(value = "根据id查询", notes = "")
    @GetMapping("/authority/getById/{id}")
    public ResultView getById(@PathVariable Integer id) {
        SysUserPO user = userService.getByUserId(id);
        return ResultView.success(user);
    }

    @ApiOperation(value = "根据id查询", notes = "")
    @GetMapping("/authority/getUserById/{id}")
    public SysUserDTO getUserById(@PathVariable Integer id) {
        SysUserPO userPO = userService.getByUserId(id);
        SysUserDTO userDTO = new SysUserDTO();
        if (null != userPO) {
            BeanUtils.copyProperties(userPO, userDTO);
        }
        return userDTO;
    }

    @ApiOperation(value = "查询用户", notes = "")
    @GetMapping("/authority/getByUserAll")
    public HashMap<Integer, SysUserDTO> getByUserAll() {
        HashMap<Integer, SysUserDTO> byUserAll = userService.getByUserAll();
        return byUserAll;
    }

    @ApiOperation(value = "新增", notes = "")
    @PostMapping("/authority_button/add")
    public ResultView add(@RequestBody SysUser model) {
        userService.add(model);
        return ResultView.success();
    }

    @ApiOperation(value = "修改", notes = "")
    @PostMapping("/authority_button/update")
    public ResultView update(@RequestBody SysUser model) {
        userService.update(model);
        return ResultView.success();
    }

    @ApiOperation(value = "修改密码", notes = "")
    @PostMapping("/authority_button/updatePassword")
    public ResultView updatePassword(@RequestBody SysUser model) {
        if (StringUtils.isNotBlank(model.getuPassword())) {
            //md5 加密
            model.setuPassword(DigestUtils.md5DigestAsHex(model.getuPassword().getBytes()));
        }
        userService.updateById(model);
        return ResultView.success();
    }

    @ApiOperation(value = "根据id判断用户是否禁用，1正常，3禁用", notes = "")
    @DeleteMapping("/authority_button/deleteById")
    public ResultView deleteById(@RequestParam int id, int status) {
        userService.deleteById(id, status);
        return ResultView.success();
    }

    @ApiOperation(value = "查看该用户名字是否重复,返回0证明该用户名不存在，1代表已经存在改用户名称", notes = "")
    @GetMapping("/authority/getByUserName/{userName}")
    public ResultView getByUserName(@PathVariable String userName) {
        Integer ifPassword = userService.getByUserName(userName);
        return ResultView.success(ifPassword);
    }

    @ApiOperation(value = "查看该用户的旧密码是否输入正确,返回0证明原密码错误，1代表原密码正确", notes = "")
    @GetMapping("/authority/getIfPassword/{id}/{password}")
    public ResultView getIfPassword(@PathVariable Integer id, @PathVariable String password) {
        Integer ifPassword = userService.getIfPassword(id, password);
        return ResultView.success(ifPassword);
    }

    @ApiOperation(value = "", notes = "")
    @PostMapping("/authority_button/temp")
    public SysUserPO add(@RequestBody SysUserPO model) {
        return model;
    }

    @ApiOperation(value = "根据depotId、recvType、recvDetails 获取相关用户信息", notes = "")
    @GetMapping("/getNoticeUserInfo")
    public Map<Integer, String> getNoticeUserInfo(@RequestParam Integer depotId, @RequestParam String recvType, @RequestParam(required = false) String recvDetails) {
        Map<Integer, String> noticeUserInfo = userService.getNoticeUserInfo(depotId, recvType, recvDetails);
        return noticeUserInfo;
    }

    @ApiOperation(value = "根据角色ID，获取用户信息（用户id，用户姓名）", notes = "")
    @GetMapping("/getUsersByRoleIds")
    public ResultView getUsersByRoleIds(@RequestParam(required = false) Integer depotId, @RequestParam String roleIds) {
        List<SysUser> usersByRoleIds = userService.getUsersByRoleIds(depotId, roleIds);
        return ResultView.success(usersByRoleIds);
    }

    @ApiOperation(value = "根据库id查询库下面的人员，不传获取所有人员", notes = "")
    @GetMapping("/authority/getAll")
    public ResultView getAll(@RequestParam(required = false) Integer depotId) {
        Map<Integer, String> all = userService.getAll(depotId);
        return ResultView.success(all);
    }

    @ApiOperation(value = "根据id查询", notes = "")
    @GetMapping("/authority/getRoleIdAll")
    public ResultView getRoleIdAll(@RequestParam String roleIds, @RequestParam(required = false) Integer depotId) {
        Map<String, List<SysUser>> roleIdAll = userService.getRoleIdAll(roleIds, depotId);
        return ResultView.success(roleIdAll);
    }

    @ApiOperation(value = "根据id查询", notes = "")
    @GetMapping("/authority/getByUserId")
    public SysUserPO getByUserId(@RequestParam("id") Integer id) {
        SysUserPO byUserId = userService.getByUserId(id);
        return byUserId;
    }
}

