package com.unissoft.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.unissoft.MyConstant;
import com.unissoft.mapper.BasicsDepotMapper;
import com.unissoft.minio.MinioUtils;
import com.unissoft.model.*;
import com.unissoft.result.PageParam;
import com.unissoft.service.BasicsDepotExtendService;
import com.unissoft.service.BasicsDepotService;
import com.unissoft.service.BasicsWarehouseService;
import org.apache.commons.lang.StringUtils;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author root
 * @since 2020-12-03
 */
@Service
public class BasicsDepotServiceImpl extends ServiceImpl<BasicsDepotMapper, BasicsDepot> implements BasicsDepotService {

    @Autowired
    BasicsDepotMapper depotMapper;

    @Autowired
    BasicsDepotExtendService depotExtendService;

    @Autowired
    BasicsWarehouseService warehouseService;

    @Autowired
    private MinioUtils minioUtils;

    @Autowired
    private BasicsDepotExtendService basicsDepotExtendService;

    @Override
    public IPage<BasicsDepot> getPage(PageParam pageParam) {
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();
        String parentId = null, pBelonging = null;
        if (!StringUtils.isEmpty(pageParam.getCondition())) {
            JSONObject object = new JSONObject(pageParam.getCondition());
            parentId = object.optString("parentId").trim();
            pBelonging = object.optString("pBelonging").trim();
        }

        Page page = null;
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        //判断是否分库
        if (StringUtils.isBlank(pBelonging)) {
            pBelonging = "0";
        }
        IPage<BasicsDepot> pageResult = depotMapper.getPage(page, parentId, pBelonging);
        return pageResult;
    }

    @Override
    public ArrayList<HashMap<String, String>> getDepotAll(Integer depotId) {
        QueryWrapper<BasicsDepot> ew = new QueryWrapper<>();
        if (depotId != null) {
            ew.eq("parent_id", depotId);
        }
        List<BasicsDepot> depots = depotMapper.selectList(ew);
        ArrayList<HashMap<String, String>> objects = Lists.newArrayList();
        if (depots.size() > 0) {
            for (BasicsDepot r : depots) {
                HashMap<String, String> map = Maps.newHashMap();
                map.put("depotCode", String.valueOf(r.getDId()));
                map.put("depotName", r.getDName());
                objects.add(map);
            }
        }
        return objects;
    }

    @Override
    public List<BasicsDepotPO> getAll(Integer depotId) {
        if (depotId == null) {
            //默认是顶级
            depotId = MyConstant.ONE;
        }
        List<BasicsDepot> depotAll = depotMapper.selectList(new QueryWrapper<>());
        List<BasicsDepotPO> depotPOS = Lists.newArrayList();
        for (BasicsDepot p : depotAll) {
            depotPOS.add(new BasicsDepotPO(p.getDId(), p.getDName(), p.getParentId(), p.getDHierarchy(), MyConstant.ZERO));
        }
        List<BasicsDepotPO> menu = createMenu(depotPOS, depotId);
        return menu;
    }

    @Override
    public List<BasicsDepotPO> getAllC(Integer depotId) {
        if (depotId == null) {
            //默认是顶级
            depotId = MyConstant.ONE;
        }
        //获取所有库点信息
        List<BasicsDepot> depotAll = depotMapper.selectList(new QueryWrapper<>());
        //获取所有仓房信息
        List<BasicsWarehouse> warehouses = warehouseService.getAll();
        //把库点和仓房装到同一个集合里面
        List<BasicsDepotPO> depotsAll = Lists.newArrayList();
        List<BasicsDepotPO> warehouseAll = Lists.newArrayList();
        for (BasicsDepot p : depotAll) {
            depotsAll.add(new BasicsDepotPO(p.getDId(), p.getDName(), p.getParentId(), p.getDHierarchy(), MyConstant.ZERO));
        }
        for (BasicsWarehouse p : warehouses) {
            warehouseAll.add(new BasicsDepotPO(p.getHId(), p.getHName(), p.getDepotId(), MyConstant.ONE, p.getCoordinate()));
        }
        //获取所有父下面的子集
        List<BasicsDepotPO> dmenu = createMenu(depotsAll, depotId);
        getCreateMenu(dmenu, warehouseAll);

        return dmenu;
    }

    @Override
    public List<BasicsDepotPO> getByHomeTypeAllC(Integer depotId, Integer homeType, String cfIds, Integer type) {
        List<BasicsDepotPO> allC = Lists.newArrayList();
        if (homeType == MyConstant.THREE) {
            QueryWrapper<BasicsDepot> ew = new QueryWrapper<>();
            if (depotId != null) {
                ew.eq("d_id", depotId);
            }
            //获取库点信息
            BasicsDepot p = depotMapper.selectOne(ew);
            if (p != null) {
                BasicsDepotPO basicsDepotPO = new BasicsDepotPO(p.getDId(), p.getDName(), p.getParentId(), p.getDHierarchy(), MyConstant.ZERO);
                //获取所有仓房信息
                List<BasicsWarehouse> warehouses = warehouseService.getByCfIds(cfIds);
                if (warehouses.size() > MyConstant.ZERO) {
                    List<BasicsDepotPO> warehouseAll = Lists.newArrayList();
                    for (BasicsWarehouse wh : warehouses) {
                        warehouseAll.add(new BasicsDepotPO(wh.getHId(), wh.getHName(), wh.getDepotId(), MyConstant.ONE, wh.getCoordinate()));
                    }
                    basicsDepotPO.setChildren(warehouseAll);
                    allC.add(basicsDepotPO);
                }
            }
        } else {
            allC = getAllC(depotId);
        }
        //type 不等于1去除仓房信息
        if (type != null && type == MyConstant.ONE) {
            removeWhether(allC);
        }
        return allC;
    }

    @Override
    public List<BasicsDepotPO> getAllC(Integer depotId, String cfIds) {
        if (depotId == null) {
            //默认是顶级
            depotId = MyConstant.ONE;
        }
        //获取所有库点信息
        List<BasicsDepot> depotAll = depotMapper.selectList(new QueryWrapper<>());
        //获取所有仓房信息
        List<BasicsWarehouse> warehouses = warehouseService.getByCfIds(cfIds);
        //把库点和仓房装到同一个集合里面
        List<BasicsDepotPO> depotsAll = Lists.newArrayList();
        List<BasicsDepotPO> warehouseAll = Lists.newArrayList();
        for (BasicsDepot p : depotAll) {
            depotsAll.add(new BasicsDepotPO(p.getDId(), p.getDName(), p.getParentId(), p.getDHierarchy(), MyConstant.ZERO));
        }
        for (BasicsWarehouse p : warehouses) {
            warehouseAll.add(new BasicsDepotPO(p.getHId(), p.getHName(), p.getDepotId(), MyConstant.ONE, p.getCoordinate()));
        }
        //获取所有父下面的子集
        List<BasicsDepotPO> dmenu = createMenu(depotsAll, depotId);
        getCreateMenu(dmenu, warehouseAll);

        return dmenu;
    }

    @Override
    public Map<Integer, String> getAllMAP(Integer depotId) {

        Map<Integer, String> returnMap = null;
        QueryWrapper<BasicsDepot> objectQueryWrapper = new QueryWrapper<>();
        if (depotId != null) {
            objectQueryWrapper.eq("parent_id", depotId);
        }
        List<BasicsDepot> depotAll = depotMapper.selectList(objectQueryWrapper);
        if (depotAll.size() > MyConstant.ZERO) {
            returnMap = Maps.newHashMap();
            for (BasicsDepot depot : depotAll) {
                returnMap.put(depot.getDId(), depot.getDName());
            }
        }
        return returnMap;
    }

    @Override
    public Map<Integer, String> getAllDirectDepot() {
        Map<Integer, String> returnMap = null;
        QueryWrapper<BasicsDepot> objectQueryWrapper = new QueryWrapper<>();
        objectQueryWrapper.eq("parent_id", MyConstant.ONE_STR);

        List<BasicsDepot> depotAll = depotMapper.selectList(objectQueryWrapper);
        if (depotAll.size() > MyConstant.ZERO) {
            returnMap = Maps.newHashMap();
            for (BasicsDepot depot : depotAll) {
                returnMap.put(depot.getDId(), depot.getDName());
            }
        }
        return returnMap;
    }

    @Transactional
    @Override
    public void uploadFiles(MultipartFile[] files, Integer depotId, String remarks) throws Exception {
        if (files != null && depotId != null) {
            ArrayList<BasicsDepotExtend> depotExtends = Lists.newArrayList();
            //获取已有图片地址
            List<BasicsDepotExtend> basicsDepotExtends = depotExtendService.getByDepotId(depotId);
            if (basicsDepotExtends.size() > 0) {
                for (BasicsDepotExtend depotExtend : basicsDepotExtends) {
                    //删除图片服务器上的图片
                    minioUtils.removeObject(MyConstant.BUCKET_NAME, depotExtend.getEPortrait());
                }
            }
            //删除图片
            depotExtendService.removeByDepotId(depotId);
            List<String> strRemarks = JSONArray.parseArray(remarks, String.class);
            for (int i = 0; i < files.length; i++) {
                MultipartFile file = files[i];
                //上传图片返回文件名称
                String str = minioUtils.uploadFile(file, MyConstant.BUCKET_NAME);
                BasicsDepotExtend depotExtend = new BasicsDepotExtend();
                depotExtend.setDepotId(depotId);
                depotExtend.setEPortrait(str);
                depotExtend.setERemark(strRemarks.get(i));
                depotExtends.add(depotExtend);
            }
            depotExtendService.saveBatch(depotExtends);
        }
    }

    @Override
    public void saveDepot(BasicsDepot model) {
        Date date = new Date();
        model.setCreateTime(date);
        if (model.getParentId() != null) {
            //设置层级关系，parentId 0是顶级父1级，1是2级，2是3级
            if (model.getParentId() == MyConstant.ONE) {
                model.setDHierarchy(MyConstant.TWO);
            } else {
                model.setDHierarchy(MyConstant.THREE);
            }
        }
        depotMapper.insert(model);
    }

    @Override
    public void update(BasicsDepot model) {
        Date date = new Date();
        model.setCreateTime(date);
        depotMapper.updateById(model);
        if (model.getDepotExtends().size() > MyConstant.ZERO) {
            basicsDepotExtendService.removeByDepotId(model.getDId());
            //批量插入库点详情信息
            basicsDepotExtendService.saveBatch(model.getDepotExtends());
        }
    }

    @Transactional
    @Override
    public void deleteById(Integer id) {
        depotMapper.deleteById(id);
        //删除子集
        basicsDepotExtendService.removeByDepotId(id);
    }

    @Override
    public void getCreateMenu(List<BasicsDepotPO> dmenu, List<BasicsDepotPO> warehouseAll) {
        for (BasicsDepotPO depot1 : dmenu) {
            //存放仓房子菜单的集合
            List<BasicsDepotPO> collect = warehouseAll.stream().filter(warehouse -> Objects.equals(warehouse.getParentCode(), depot1.getDepotCode())).collect(Collectors.toList());
            if (depot1.getChildren() != null) {
                List<BasicsDepotPO> children = depot1.getChildren();
                for (BasicsDepotPO chind : children) {
                    List<BasicsDepotPO> collect2 = warehouseAll.stream().filter(warehouse -> Objects.equals(warehouse.getParentCode(), chind.getDepotCode())).collect(Collectors.toList());
                    if (chind.getChildren() != null) {
                        List<BasicsDepotPO> chindChildren = chind.getChildren();
                        for (BasicsDepotPO chind2 : chindChildren) {
                            //存放仓房子菜单的集合
                            List<BasicsDepotPO> listChild = warehouseAll.stream().filter(warehouse -> Objects.equals(warehouse.getParentCode(), chind2.getDepotCode())).collect(Collectors.toList());
                            chind2.setChildren(listChild);
                        }
                        collect2.addAll(chind.getChildren());
                        chind.setChildren(collect2);
                    } else {
                        chind.setChildren(collect2);
                    }
                }
                collect.addAll(depot1.getChildren());
                depot1.setChildren(collect);
            } else {
                depot1.setChildren(collect);
            }
        }
    }

    @Override
    public List<BasicsDepot> getAllDepot() {
        List<BasicsDepot> depotAll = depotMapper.selectList(new QueryWrapper<>());
        return depotAll;
    }

    @Override
    public List<BasicsDepotAL> getAllLONAndLAT(Integer homeType, Integer depotId) {
        return depotMapper.getAllLONAndLAT(homeType, depotId);
    }

    private List<BasicsDepotPO> createMenu(List<BasicsDepotPO> mp01_menus, Integer depotId) {
        ArrayList<BasicsDepotPO> rootMenu = new ArrayList<>();

        //查询根节点数据
        for (BasicsDepotPO menu : mp01_menus) {
            if (menu.getDepotCode().equals(depotId)) {
                rootMenu.add(menu);
            }
        }

        //获取根节点下的子节点
        for (BasicsDepotPO menu : rootMenu) {
            List<BasicsDepotPO> child = getChild(menu.getDepotCode(), mp01_menus);
            menu.setChildren(child);
        }
        return rootMenu;
    }

    private List<BasicsDepotPO> getChild(Integer menuid, List<BasicsDepotPO> allmp01_menus) {
        //存放子菜单的集合
        ArrayList<BasicsDepotPO> listChild = new ArrayList<>();
        for (BasicsDepotPO menu : allmp01_menus) {
            if (menu.getParentCode().equals(menuid)) {
                listChild.add(menu);
            }
        }
        //递归：
        for (BasicsDepotPO menu : listChild) {
            menu.setChildren(getChild(menu.getDepotCode(), allmp01_menus));
        }
        if (listChild.size() == 0) {
            return null;
        }
        return listChild;
    }

    /**
     * 去除仓房信息
     *
     * @param allC
     */
    private void removeWhether(List<BasicsDepotPO> allC) {
        if (allC.size() > MyConstant.ZERO) {
            allC.stream().forEach(a -> {
                if (a.getChildren() != null) {
                    List<BasicsDepotPO> collect = a.getChildren().stream().filter(t -> t.getIsWhether() != MyConstant.ONE).collect(Collectors.toList());
                    a.setChildren(collect);
                    collect.stream().forEach(b -> {
                        if (b.getChildren() != null) {
                            List<BasicsDepotPO> collect1 = b.getChildren().stream().filter(j -> j.getIsWhether() != MyConstant.ONE).collect(Collectors.toList());
                            b.setChildren(collect1);
                            collect1.stream().forEach(c -> {
                                if (c.getChildren() != null) {
                                    List<BasicsDepotPO> collect2 = c.getChildren().stream().filter(x -> x.getIsWhether() != MyConstant.ONE).collect(Collectors.toList());
                                    c.setChildren(collect2);
                                }
                            });
                        }
                    });
                }
            });
        }
    }
}
