package com.unissoft.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Maps;
import com.unissoft.MyConstant;
import com.unissoft.mapper.BasicsEnumMapper;
import com.unissoft.model.BasicsEnum;
import com.unissoft.model.BasicsEnumPO;
import com.unissoft.service.BasicsEnumService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author root
 * @since 2021-01-06
 */
@Service
public class BasicsEnumServiceImpl extends ServiceImpl<BasicsEnumMapper, BasicsEnum> implements BasicsEnumService {

    @Autowired
    private BasicsEnumMapper enumMapper;

    @Override
    public List<BasicsEnumPO> getEnumAll() {
        List<BasicsEnumPO> basicsEnums = enumMapper.getAll();
        List<BasicsEnumPO> menu = createMenu(basicsEnums, MyConstant.ZERO);
        return menu;
    }

    @Override
    public HashMap<String, Object> getAll() {
        List<BasicsEnumPO> basicsEnums = enumMapper.getAll();
        Map<Integer, List<BasicsEnumPO>> resultMap = new HashMap<>();
        List<Integer> parentIds = new ArrayList<Integer>();
        for (BasicsEnumPO be : basicsEnums) {
            if (!parentIds.contains(be.getParentId())) {
                parentIds.add(be.getParentId());
            }
        }

        for (Integer parentId : parentIds) {
            List<BasicsEnumPO> children = this.getChildren(basicsEnums, parentId);
            resultMap.put(parentId, children);
        }
        //单个map
        HashMap<Integer, String> enumMap = Maps.newHashMap();
        for (BasicsEnumPO e : basicsEnums) {
            enumMap.put(e.getEnumId(), e.getEnumName());
        }

        HashMap<String, Object> mapAll = Maps.newHashMap();
        mapAll.put("parentMap", resultMap);
        mapAll.put("enumMap", enumMap);
        return mapAll;
    }

    @Override
    public void deleteById(Integer id) {
        QueryWrapper<BasicsEnum> qw = new QueryWrapper<>();
        qw.eq("enum_id", id);
        BasicsEnum basicsEnum = enumMapper.selectOne(qw);
        if (basicsEnum != null) {
            basicsEnum.setDelFlag(1);
            enumMapper.updateById(basicsEnum);
        }
    }

    @Override
    public List<BasicsEnum> getOnlyChildData(Integer parentId) {
        if (null == parentId) {
            return Collections.emptyList();
        }
        return enumMapper.selectOnlyChildData(parentId);
    }

    private List<BasicsEnumPO> getChildren(List<BasicsEnumPO> list, Integer parentId) {
        List<BasicsEnumPO> children = new ArrayList<BasicsEnumPO>();
        for (BasicsEnumPO be : list) {
            //遍历出父id等于参数的id，add进子节点集合
            if (be.getParentId().equals(parentId)) {
                children.add(be);
            }
        }
        return children;
    }


    private List<BasicsEnumPO> createMenu(List<BasicsEnumPO> mp01_menus, Integer enumId) {
        ArrayList<BasicsEnumPO> rootMenu = new ArrayList<>();

//查询根节点数据
        for (BasicsEnumPO menu : mp01_menus) {
            if (menu.getParentId().equals(enumId)) {
                rootMenu.add(menu);
            }
        }

//获取根节点下的子节点
        for (BasicsEnumPO menu : rootMenu) {
            List<BasicsEnumPO> child = getChild(menu.getEnumId(), mp01_menus);
            menu.setChildren(child);
        }
        return rootMenu;
    }

    private List<BasicsEnumPO> getChild(Integer menuid, List<BasicsEnumPO> allmp01_menus) {
//存放子菜单的集合
        ArrayList<BasicsEnumPO> listChild = new ArrayList<>();
        for (BasicsEnumPO menu : allmp01_menus) {
            if (menu.getParentId().equals(menuid)) {
                listChild.add(menu);
            }
        }
//递归：
        for (BasicsEnumPO menu : listChild) {
            menu.setChildren(getChild(menu.getEnumId(), allmp01_menus));
        }
        if (listChild.size() == 0) {
            return null;
        }
        return listChild;
    }
}
