package com.unissoft.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.unissoft.MyConstant;
import com.unissoft.mapper.BasicsWarehouseMapper;
import com.unissoft.model.BasicsDepot;
import com.unissoft.model.BasicsDepotPO;
import com.unissoft.model.BasicsWarehouse;
import com.unissoft.model.BasicsWarehousePO;
import com.unissoft.result.PageParam;
import com.unissoft.service.BasicsEnumService;
import com.unissoft.service.BasicsWarehouseService;
import org.apache.commons.lang.StringUtils;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author root
 * @since 2020-12-03
 */
@Service
public class BasicsWarehouseServiceImpl extends ServiceImpl<BasicsWarehouseMapper, BasicsWarehouse> implements BasicsWarehouseService {

    @Autowired
    private BasicsDepotServiceImpl depotService;

    @Autowired
    private BasicsWarehouseMapper warehouseMapper;
    @Autowired
    private BasicsDepotServiceRecursion depotServiceRecursion;
    @Autowired
    private BasicsEnumService basicsEnumService;

    @Override
    public Page<BasicsWarehouse> getPage(PageParam pageParam) {
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();
        QueryWrapper<BasicsWarehouse> ew = new QueryWrapper<>();
        if (!StringUtils.isEmpty(pageParam.getCondition())) {
            JSONObject object = new JSONObject(pageParam.getCondition());
            String depotId = object.optString("depotId").trim();
            if (!StringUtils.isEmpty(depotId)) {
                ew.eq("depot_id", depotId);
            }
        }
//        ew.orderByDesc("create_time");
        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        Page<BasicsWarehouse> pageResult = warehouseMapper.selectPage(page, ew);
        return pageResult;
    }

    @Override
    public ArrayList<HashMap<String, String>> getDepotAll(Integer depotId) {
        QueryWrapper<BasicsWarehouse> ew = new QueryWrapper<>();
        ew.eq("depot_id", depotId);
        ew.orderByAsc("h_order");
        List<BasicsWarehouse> warehouses = warehouseMapper.selectList(ew);
        ArrayList<HashMap<String, String>> objects = Lists.newArrayList();
        if (warehouses.size() > 0) {
            for (BasicsWarehouse r : warehouses) {
                HashMap<String, String> map = Maps.newHashMap();
                map.put("houseCode", String.valueOf(r.getHId()));
                map.put("houseName", r.getHName());
                objects.add(map);
            }
        }
        return objects;
    }

    @Override
    public Map<Integer, String> getAllMAP(Integer depotId) {
        Map<Integer, String> returnMap = null;
        QueryWrapper<BasicsWarehouse> objectQueryWrapper = new QueryWrapper<>();
        if (depotId != null) {
            objectQueryWrapper.eq("depot_id", depotId);
        }
        objectQueryWrapper.orderByAsc("h_order");
        List<BasicsWarehouse> warehouses = warehouseMapper.selectList(objectQueryWrapper);
        if (warehouses.size() > MyConstant.ZERO) {
            returnMap = Maps.newHashMap();
            for (BasicsWarehouse warehouse : warehouses) {
                returnMap.put(warehouse.getHId(), warehouse.getHName());
            }
        }
        return returnMap;
    }

    @Override
    public List<BasicsWarehouse> getAll() {
        QueryWrapper<BasicsWarehouse> ew = new QueryWrapper<>();
        ew.orderByAsc("h_order");
        List<BasicsWarehouse> warehouses = warehouseMapper.selectList(ew);
        return warehouses;
    }

    @Override
    public Map<String, List<BasicsWarehousePO>> getPercent(BasicsWarehouse basicsWarehouse) {
        // 调用递归方法获取所有仓房id
        List<Integer> allDepotChild = depotServiceRecursion.getAllDepotChild(basicsWarehouse.getDepotId());
        // 根据仓房id查询,获取(粮食品种,储粮状态,仓房状态)
        List<BasicsWarehouse> percent = warehouseMapper.getPercent(allDepotChild);
        // 储粮状态分组统计
        Map<Integer, List<BasicsWarehouse>> collect = percent.stream().collect(Collectors.groupingBy(BasicsWarehouse::getStorageStatus));
        // 储粮状态(storageStatus),仓房状态(warehouseStatus),粮食品种(grainVarieties) 数组.
        List<BasicsWarehousePO> objects = Lists.newArrayList();
        // 返回map
        Map<String, List<BasicsWarehousePO>> listHashMap = Maps.newHashMap();
        // 获取字典
        Map enumMap = (Map) basicsEnumService.getAll().get("enumMap");
        collect.forEach((k, v) -> {
            BasicsWarehousePO basicsWarehousePO = new BasicsWarehousePO();
            // 获取字典对应值
            String o = enumMap.get(k).toString();
            basicsWarehousePO.setStorageStatus(o);
            basicsWarehousePO.setCount(v.size());
            objects.add(basicsWarehousePO);
        });
        listHashMap.put("storageStatus", objects);
        // 仓房状态分组统计
        List<BasicsWarehousePO> objects1 = Lists.newArrayList();
        Map<Integer, List<BasicsWarehouse>> collect1 = percent.stream().collect(Collectors.groupingBy(BasicsWarehouse::getWarehouseStatus));
        collect1.forEach((k, v) -> {
            BasicsWarehousePO basicsWarehousePO = new BasicsWarehousePO();
            // 获取字典对应值
            String o = enumMap.get(k).toString();
            basicsWarehousePO.setWarehouseStatus(o);
            basicsWarehousePO.setCount(v.size());
            objects1.add(basicsWarehousePO);
        });
        listHashMap.put("warehouseStatus", objects1);
        // 粮食品种分组统计
        List<BasicsWarehousePO> objects2 = Lists.newArrayList();
        Map<Integer, List<BasicsWarehouse>> collect2 = percent.stream().collect(Collectors.groupingBy(BasicsWarehouse::getGrainVarieties));
        collect2.forEach((k, v) -> {
            BasicsWarehousePO basicsWarehousePO = new BasicsWarehousePO();
            // 获取字典对应值
            String o = enumMap.get(k).toString();
            basicsWarehousePO.setGrainVarieties(o);
            basicsWarehousePO.setCount(v.size());
            objects2.add(basicsWarehousePO);
        });
        listHashMap.put("grainVarieties", objects2);
        return listHashMap;
    }

    @Override
    public List<BasicsWarehouse> getPictureAllMApicture(Integer depotId) {

        QueryWrapper<BasicsWarehouse> objectQueryWrapper = new QueryWrapper<>();
        if (depotId != null) {
            objectQueryWrapper.eq("depot_id", depotId);
        }
        objectQueryWrapper.orderByAsc("h_order");
        List<BasicsWarehouse> warehouses = warehouseMapper.selectList(objectQueryWrapper);
        return warehouses;
    }

    @Override
    public List<BasicsWarehouse> getByCfIds(String cfIds) {
        QueryWrapper<BasicsWarehouse> objectQueryWrapper = new QueryWrapper<>();
        if (!StringUtils.isBlank(cfIds)){
            objectQueryWrapper.in("h_id",cfIds.split(","));
        }
        objectQueryWrapper.orderByAsc("h_order");
        List<BasicsWarehouse> warehouses = warehouseMapper.selectList(objectQueryWrapper);
        return warehouses;
    }

    @Override
    public List<BasicsDepotPO> getByDepotIDAllC(Integer depotId, String cfIds) {
        List<BasicsDepotPO> allC = depotService.getAllC(depotId, cfIds);
        return allC;
    }
}
