package com.unissoft.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Maps;
import com.unissoft.MyConstant;
import com.unissoft.mapper.MCompanyManagementMapper;
import com.unissoft.model.MCompanyManagement;
import com.unissoft.model.SysUserPO;
import com.unissoft.result.PageParam;
import com.unissoft.service.MCompanyManagementService;
import com.unissoft.service.SysPersonnelService;
import com.unissoft.utils.UserUtils;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author root
 * @since 2021-01-28
 */
@Service
public class MCompanyManagementServiceImpl extends ServiceImpl<MCompanyManagementMapper, MCompanyManagement> implements MCompanyManagementService {

    @Autowired
    private MCompanyManagementMapper managementMapper;

    @Autowired
    private SysPersonnelService personnelService;

    @Override
    public Page<MCompanyManagement> getPage(PageParam pageParam) {
        int pageIndex = pageParam.getPageIndex();
        int pageSize = pageParam.getPageSize();
        QueryWrapper<MCompanyManagement> ew = new QueryWrapper<>();
        if (!StringUtils.isEmpty(pageParam.getCondition())) {
            JSONObject object = new JSONObject(pageParam.getCondition());
            String mType = object.optString("mType");
            String id = object.optString("id");
            String depotId = object.optString("depotId");
            if (!StringUtils.isEmpty(mType)) {
                ew.eq("m_type", mType);
            }
            if (!StringUtils.isEmpty(id)) {
                ew.eq("id", id);
            }
            if (!StringUtils.isEmpty(depotId)) {
                ew.eq("depot_id", depotId);
            }
        }
        ew.orderByDesc("create_time");
        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        Page<MCompanyManagement> pageResult = managementMapper.selectPage(page, ew);
        return pageResult;
    }

    @Transactional
    @Override
    public void saveGrain(MCompanyManagement model) {
        model.setCreateTime(new Date());
        managementMapper.insert(model);
        //人员id不等于null标注该人员是库外管理人员
        if (model.getMType() != null && model.getResidentPersonnelId() != null) {
            personnelService.updateByPerId(model.getResidentPersonnelId(), model.getMType(), MyConstant.ZERO);
        }
    }

    @Transactional
    @Override
    public void updateRole(MCompanyManagement model) {
        model.setUpdateTime(new Date());
        managementMapper.updateById(model);
        //人员id不等于null标注该人员是库外管理人员
        if (model.getMType() != null && model.getResidentPersonnelId() != null) {
            personnelService.updateByPerId(model.getResidentPersonnelId(), model.getMType(), MyConstant.ONE);
        }
    }

    @Transactional
    @Override
    public void deleteById(Integer id) {
        MCompanyManagement management = managementMapper.selectById(id);
        if (management != null) {
            //删除人员关联关系
            personnelService.updateByPerId(management.getResidentPersonnelId(), management.getMType(), MyConstant.TWO);
            managementMapper.deleteById(id);
        }
    }

    @Override
    public Map<Integer, String> getAll(Integer depotId) {
        Map<Integer, String> returnMap = null;
        QueryWrapper<MCompanyManagement> objectQueryWrapper = new QueryWrapper<>();
        if (depotId != null) {
            objectQueryWrapper.eq("depot_id", depotId);
        }

        List<MCompanyManagement> depotAll = managementMapper.selectList(objectQueryWrapper);
        if (depotAll.size() > MyConstant.ZERO) {
            returnMap = Maps.newHashMap();
            for (MCompanyManagement pan : depotAll) {
                returnMap.put(pan.getId(), pan.getEnterpriseName());
            }
        }
        return returnMap;
    }

    @Override
    public MCompanyManagement getCompanyName() {
        MCompanyManagement management = null;
        SysUserPO currUserInfo = UserUtils.getCurrUserInfo();
        if (currUserInfo != null) {
            QueryWrapper<MCompanyManagement> objectQueryWrapper = new QueryWrapper<>();
            objectQueryWrapper.eq("resident_personnel_id", currUserInfo.getPersonnelId());
            List<MCompanyManagement> mCompanyManagements = managementMapper.selectList(objectQueryWrapper);
            if (mCompanyManagements.size() > MyConstant.ZERO) {
                management = mCompanyManagements.get(MyConstant.ZERO);
            }
        }
        return management;
    }
}
