package com.unissoft.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.MyConstant;
import com.unissoft.mapper.MEntrustRentingGrainMapper;
import com.unissoft.model.MCompanyManagement;
import com.unissoft.model.MEntrustRentingGrain;
import com.unissoft.result.PageParam;
import com.unissoft.service.MCompanyManagementService;
import com.unissoft.service.MEntrustRentingGrainService;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.Date;

/**
 * <p>
 * 托管，租仓粮情 服务实现类
 * </p>
 *
 * @author root
 * @since 2021-01-27
 */
@Service
public class MEntrustRentingGrainServiceImpl extends ServiceImpl<MEntrustRentingGrainMapper, MEntrustRentingGrain> implements MEntrustRentingGrainService {

    @Autowired
    private MEntrustRentingGrainMapper grainMapper;

    @Autowired
    private MCompanyManagementService managementService;

    @Override
    public Page<MEntrustRentingGrain> getPage(PageParam pageParam) {
        int pageIndex = pageParam.getPageIndex();
        int pageSize = pageParam.getPageSize();
        QueryWrapper<MEntrustRentingGrain> ew = new QueryWrapper<>();

        if (!StringUtils.isEmpty(pageParam.getCondition())) {
            JSONObject object = new JSONObject(pageParam.getCondition());
            String mType = object.optString("mType");
            String depotId = object.optString("depotId");
            String companyId = object.optString("companyId");
            String statTime = object.optString("statTime");
            String endTime = object.optString("endTime");
            if (!StringUtils.isEmpty(mType)) {
                ew.eq("m_type", mType);
            }
            if (!StringUtils.isEmpty(depotId)) {
                ew.eq("depot_id", depotId);
            }
            if (!StringUtils.isEmpty(companyId)) {
                ew.eq("company_id", companyId);
            }
            if (!StringUtils.isEmpty(statTime) && !StringUtils.isEmpty(endTime)) {
                statTime += MyConstant.STR_TIME;
                endTime += MyConstant.STR_END;
                ew.between("create_time", statTime, endTime);
            }
        }
        ew.orderByDesc("create_time");
        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        Page<MEntrustRentingGrain> pageResult = grainMapper.selectPage(page, ew);
        return pageResult;
    }

    @Override
    public void saveGrain(MEntrustRentingGrain model) {
        //拼接异常项名称
        String exception = this.getException(model);
        if (!StringUtils.isEmpty(exception)) {
            model.setMException(exception);
            // 有无异常(0绿色，1黄色)
            model.setIsException(MyConstant.ONE);
        } else {
            model.setIsException(MyConstant.ZERO);
        }
        //状态 0进行中，1已完成
        Integer mState = this.getMState(model);
        if (mState != null) {
            model.setMState(mState);
        }
        model.setMCheckType(MyConstant.M_CHECK_TYPE);
        model.setCreateTime(new Date());
        //获取公司信息
        MCompanyManagement companyName = managementService.getCompanyName();
        if (companyName != null) {
            model.setCompanyId(companyName.getId());
            model.setCompanyName(companyName.getEnterpriseName());
        }
        grainMapper.insert(model);
    }

    @Override
    public void updateRole(MEntrustRentingGrain model) {
        //拼接异常项名称
        String exception = this.getException(model);
        if (!StringUtils.isEmpty(exception)) {
            model.setMException(exception);
            // 有无异常(0绿色，1黄色)
            model.setIsException(MyConstant.ONE);
        } else {
            model.setMException("");
            model.setIsException(MyConstant.ZERO);
        }
        //状态 0进行中，1已完成
        Integer mState = this.getMState(model);
        if (mState != null) {
            model.setMState(mState);
        }
        model.setMCheckType(MyConstant.M_CHECK_TYPE);
        model.setCreateTime(new Date());
        MCompanyManagement companyName = managementService.getCompanyName();
        if (companyName != null) {
            model.setCompanyId(companyName.getId());
            model.setCompanyName(companyName.getEnterpriseName());
        }
        grainMapper.updateById(model);
    }

    @Override
    public void deleteById(Integer id) {
        grainMapper.deleteById(id);
    }

    /**
     * @param model
     * @return 拼接异常项名称
     */
    private String getException(MEntrustRentingGrain model) {
        StringBuilder str = new StringBuilder();
        //异常项那里显示名称：虫害异常、水分检测异常、粮情异常; 拼接异常项名称
        if (model.getMInsect() != null && model.getMInsect() == MyConstant.ZERO) {
            // 虫害异常
            str.append(MyConstant.M_INSECT);
        }
        if (model.getWaterContent() != null && model.getWaterContent() == MyConstant.ZERO) {
            // 水分检测是否正常
            str.append(MyConstant.WATER_CONTENT);
        }
        if (model.getGrainCondition() != null && model.getGrainCondition() == MyConstant.ZERO) {
            // 异常粮情
            str.append(MyConstant.GRAIN_CONDITION);
        }
        String s = str.toString();
        if (s.length() > MyConstant.TWO) {
            s = s.substring(MyConstant.ZERO, s.length() - MyConstant.ONE);
        }
        return s;
    }

    /**
     * @param model
     * @return 状态 0进行中，1已完成
     */
    private Integer getMState(MEntrustRentingGrain model) {
        Integer state = null;
        if (model.getPrintGrainResults() != null && model.getOnLight() != null &&
                model.getDoubleEntry() != null && model.getGasConcentration() != null &&
                model.getMFloodlight() != null && model.getColorSmell() != null &&
                model.getMInsect() != null && model.getWaterContent() != null &&
                model.getGrainCondition() != null && model.getLockDoor() != null &&
                model.getOffLight() != null && model.getGrainWork() != null &&
                model.getKeepLog() != null && model.getGrainAnalysi() != null) {
            state = MyConstant.ONE;
        } else {
            state = MyConstant.ZERO;
        }
        return state;
    }
}
