package com.unissoft.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.MyConstant;
import com.unissoft.mapper.MSafetyInspectionMapper;
import com.unissoft.model.MCompanyManagement;
import com.unissoft.model.MSafetyInspection;
import com.unissoft.result.PageParam;
import com.unissoft.service.MCompanyManagementService;
import com.unissoft.service.MSafetyInspectionService;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.Date;

/**
 * <p>
 * 安全巡查表(委托,租仓) 服务实现类
 * </p>
 *
 * @author root
 * @since 2021-01-28
 */
@Service
public class MSafetyInspectionServiceImpl extends ServiceImpl<MSafetyInspectionMapper, MSafetyInspection> implements MSafetyInspectionService {

    @Autowired
    private MSafetyInspectionMapper inspectionMapper;

    @Autowired
    private MCompanyManagementService managementService;

    @Override
    public Page<MSafetyInspection> getPage(PageParam pageParam) {
        int pageIndex = pageParam.getPageIndex();
        int pageSize = pageParam.getPageSize();
        QueryWrapper<MSafetyInspection> ew = new QueryWrapper<>();

        if (!StringUtils.isEmpty(pageParam.getCondition())) {
            JSONObject object = new JSONObject(pageParam.getCondition());
            String mType = object.optString("mType");
            String companyId = object.optString("companyId");
            String depotId = object.optString("depotId");
            String statTime = object.optString("statTime");
            String endTime = object.optString("endTime");
            if (!StringUtils.isEmpty(mType)) {
                ew.eq("m_type", mType);
            }
            if (!StringUtils.isEmpty(depotId)) {
                ew.eq("depot_id", depotId);
            }
            if (!StringUtils.isEmpty(companyId)) {
                ew.eq("company_id", companyId);
            }
            if (!StringUtils.isEmpty(statTime) && !StringUtils.isEmpty(endTime)) {
                statTime += MyConstant.STR_TIME;
                endTime += MyConstant.STR_END;
                ew.between("create_time", statTime, endTime);
            }
        }
        ew.orderByDesc("create_time");
        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        Page<MSafetyInspection> pageResult = inspectionMapper.selectPage(page, ew);
        return pageResult;
    }

    @Override
    public void saveGrain(MSafetyInspection model) {
        model.setCreateTime(new Date());
        model.setSType(MyConstant.SAFETY_INSPECTION);
        Integer mState = this.getMState(model);
        if (mState != null) {
            model.setMState(mState);
        }
        MCompanyManagement companyName = managementService.getCompanyName();
        if (companyName != null) {
            model.setCompanyId(companyName.getId());
            model.setCompanyName(companyName.getEnterpriseName());
        }
        inspectionMapper.insert(model);
    }

    @Override
    public void updateRole(MSafetyInspection model) {
        model.setCreateTime(new Date());
        model.setSType(MyConstant.SAFETY_INSPECTION);
        Integer mState = this.getMState(model);
        if (mState != null) {
            model.setMState(mState);
        }
        MCompanyManagement companyName = managementService.getCompanyName();
        if (companyName != null) {
            model.setCompanyId(companyName.getId());
            model.setCompanyName(companyName.getEnterpriseName());
        }
        inspectionMapper.updateById(model);
    }

    @Override
    public void deleteById(Integer id) {
        inspectionMapper.deleteById(id);
    }

    /**
     * @param model
     * @return 状态 0进行中，1已完成
     */
    private Integer getMState(MSafetyInspection model) {
        Integer state = null;
        if (model.getSafetyOfficer() != null && model.getSafetyArchive() != null &&
                model.getDailyInspection() != null && model.getDailyOnlineMonitoring() != null &&
                model.getDiscoverProblem() != null && model.getFillReport() != null &&
                model.getProblemElimination() != null && model.getRegisterAccount() != null) {
            state = MyConstant.ONE;
        } else {
            state = MyConstant.ZERO;
        }
        return state;
    }
}
