package com.unissoft.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Maps;
import com.unissoft.MyConstant;
import com.unissoft.mapper.ProjectMapper;
import com.unissoft.model.ProjectModel;
import com.unissoft.model.SysDepartment;
import com.unissoft.result.PageParam;
import com.unissoft.service.ProjectService;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 项目（如公司里某个项目） 服务实现类
 * </p>
 *
 * @author wsm
 * @since 2019-07-16
 */
@Service
public class ProjectServicel extends ServiceImpl<ProjectMapper, ProjectModel> implements ProjectService {

    @Autowired
    private ProjectMapper projectMapper;

    @Override
    public Page<ProjectModel> getPage(PageParam pageParam) {
        int pageIndex = pageParam.getPageIndex();
        int pageSize = pageParam.getPageSize();
        QueryWrapper<ProjectModel> ew = new QueryWrapper<>();
        if (!StringUtils.isEmpty(pageParam.getCondition())){
            JSONObject object = new JSONObject();
            String name = object.optString("name").trim();
            String code = object.optString("code");
            String regionId = object.optString("regionId");
            String isEnable = object.optString("isEnable");
            if (!StringUtils.isEmpty(name)) {
                ew.like("name", name);
            }
            if (!StringUtils.isEmpty(code)) {
                ew.eq("code", code);
            }
            if (!StringUtils.isEmpty(regionId)) {
                ew.eq("region_id", regionId);
            }
            if (!StringUtils.isEmpty(isEnable)) {
                ew.eq("isEnable", isEnable);
            }
        }
        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        Page<ProjectModel> pageResult = projectMapper.selectPage(page, ew);
        return pageResult;
    }

    @Override
    public Map<Integer, String> getAllMAP() {
        Map<Integer, String> returnMap = null;
        List<ProjectModel> departments = projectMapper.selectList(new QueryWrapper<>());
        if (departments.size() > MyConstant.ZERO) {
            returnMap = Maps.newHashMap();
            for (ProjectModel department : departments) {
                returnMap.put(department.getId().intValue(), department.getName());
            }
        }
        return returnMap;
    }
}
