package com.unissoft.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.unissoft.MyConstant;
import com.unissoft.mapper.SysDepartmentMapper;
import com.unissoft.model.BasicsDepot;
import com.unissoft.model.SysDepartment;
import com.unissoft.model.SysUser;
import com.unissoft.result.PageParam;
import com.unissoft.service.SysDepartmentService;
import org.apache.commons.lang.StringUtils;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author root
 * @since 2020-12-04
 */
@Service
public class SysDepartmentServiceImpl extends ServiceImpl<SysDepartmentMapper, SysDepartment> implements SysDepartmentService {

    @Autowired
    private SysDepartmentMapper departmentMapper;

    @Override
    public Page<SysDepartment> getPage(PageParam pageParam) {
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();
        QueryWrapper<SysDepartment> ew = new QueryWrapper<>();
        if (!StringUtils.isEmpty(pageParam.getCondition())) {
            JSONObject object = new JSONObject(pageParam.getCondition());
            String dName = object.optString("dName").trim();
            if (!StringUtils.isEmpty(dName)) {
                ew.like("d_name", dName);
            }
        }
        ew.orderByDesc("create_time");
        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        Page<SysDepartment> pageResult = departmentMapper.selectPage(page, ew);
        return pageResult;
    }

    @Override
    public ArrayList<HashMap<String, String>> getDepartmentAll() {
        List<SysDepartment> depots = departmentMapper.selectList(new QueryWrapper<>());
        ArrayList<HashMap<String, String>> objects = Lists.newArrayList();
        if (depots.size() > 0) {
            for (SysDepartment r : depots) {
                HashMap<String, String> map = Maps.newHashMap();
                map.put("departmentCode", String.valueOf(r.getDId()));
                map.put("departmentName", r.getDName());
                objects.add(map);
            }
        }
        return objects;
    }

    @Override
    public List<SysDepartment> getAll() {
        List<SysDepartment> depots = departmentMapper.selectList(new QueryWrapper<>());
        return depots;
    }

    @Override
    public Map<Integer, String> getAllMAP() {
        Map<Integer, String> returnMap = null;
        List<SysDepartment> departments = departmentMapper.selectList(new QueryWrapper<>());
        if (departments.size() > MyConstant.ZERO) {
            returnMap = Maps.newHashMap();
            for (SysDepartment department : departments) {
                returnMap.put(department.getDId(), department.getDName());
            }
        }
        return returnMap;
    }

    @Override
    public Integer getByDepName(String depName) {
        QueryWrapper<SysDepartment> qw = new QueryWrapper<>();
        qw.eq("d_name", depName);
        Integer integer = departmentMapper.selectCount(qw);
        return integer;
    }
}
