package com.unissoft.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.MyConstant;
import com.unissoft.mapper.SysLogMapper;
import com.unissoft.model.SysLog;
import com.unissoft.result.PageParam;
import com.unissoft.service.SysLogService;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

/**
 * <p>
 * 系统日志 服务实现类
 * </p>
 *
 * @author root
 * @since 2021-01-21
 */
@Service
public class SysLogServiceImpl extends ServiceImpl<SysLogMapper, SysLog> implements SysLogService {

    @Autowired
    private SysLogMapper logMapper;


    @Override
    public Page<SysLog> getPage(PageParam pageParam) {
        int pageIndex = pageParam.getPageIndex();
        int pageSize = pageParam.getPageSize();
        QueryWrapper<SysLog> ew = new QueryWrapper<>();
        if (!StringUtils.isEmpty(pageParam.getCondition())) {
            JSONObject object = new JSONObject(pageParam.getCondition());
            String depotId = object.optString("depotId").trim();
            if (!StringUtils.isEmpty(depotId)) {
                ew.eq("depot_id", depotId);
            }
        }
        ew.orderByDesc("create_time");
        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        Page<SysLog> pageResult = logMapper.selectPage(page, ew);
        //封装数据返回
        if (pageResult.getRecords() != null) {
            for (SysLog log : pageResult.getRecords()) {
                String requestUrl = log.getRequestUrl();
                if (requestUrl != null) {
                    String[] split = requestUrl.split("/");
                    if (split.length > MyConstant.TWO) {
                        log.setRequestAction(split[MyConstant.TWO]);
                    }
                    if (split.length > MyConstant.FOUR) {
                        log.setRequestMethod(split[MyConstant.FOUR].split("\\?")[MyConstant.ZERO]);
                    }
                }
            }
        }
        return pageResult;
    }

}
