package com.unissoft.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.unissoft.MyConstant;
import com.unissoft.mapper.SysPersonnelMapper;
import com.unissoft.minio.MinioUtils;
import com.unissoft.model.SysDepartment;
import com.unissoft.model.SysPersonnel;
import com.unissoft.model.SysPersonnelPO;
import com.unissoft.result.PageParam;
import com.unissoft.service.SysDepartmentService;
import com.unissoft.service.SysPersonnelService;
import org.apache.commons.lang.StringUtils;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author root
 * @since 2020-12-01
 */
@Service
public class SysPersonnelServiceImpl extends ServiceImpl<SysPersonnelMapper, SysPersonnel> implements SysPersonnelService {

    @Autowired
    private SysPersonnelMapper personnelMapper;

    @Autowired
    private MinioUtils minioUtils;

    @Autowired
    private SysDepartmentService departmentService;

    @Override
    public Page<SysPersonnel> getPage(PageParam pageParam) {
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();
        QueryWrapper<SysPersonnel> ew = new QueryWrapper<>();
        if (!StringUtils.isEmpty(pageParam.getCondition())) {
            JSONObject object = new JSONObject(pageParam.getCondition());
            String name = object.optString("uName").trim();
            String depotId = object.optString("depotId").trim();
            if (!StringUtils.isEmpty(name)) {
                ew.like("u_name", name);
            }
            if (!StringUtils.isEmpty(depotId)) {
                ew.eq("depot_id", depotId);
            }
        }
        ew.eq("u_status", 1);
        ew.orderByDesc("create_time");
        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        Page<SysPersonnel> pageResult = personnelMapper.selectPage(page, ew);
        return pageResult;
    }

    @Override
    public Map<String, Object> getByDepartmentPage(PageParam pageParam) {
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();
        QueryWrapper<SysPersonnel> ew = new QueryWrapper<>();
        String depotId = null;
        if (!StringUtils.isEmpty(pageParam.getCondition())) {
            JSONObject object = new JSONObject(pageParam.getCondition());
            //所属部门
            String departmentId = object.optString("departmentId").trim();
            //库点id
            depotId = object.optString("depotId").trim();
            if (!StringUtils.isEmpty(departmentId)) {
                ew.eq("department_id", departmentId);
            }
            if (!StringUtils.isEmpty(depotId)) {
                ew.eq("depot_id", depotId);
            }
        }
        ew.eq("u_status", 1);
        ew.orderByDesc("create_time");
        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        Page<SysPersonnel> pageResult = personnelMapper.selectPage(page, ew);
        HashMap<String, Object> returnMAP = Maps.newHashMap();
        returnMAP.put("pageResult", pageResult);

        //根据固定部门，固定专业统计查询
        List<SysPersonnel> collect = personnelMapper.getByDepotId(Integer.valueOf(depotId));
        if (collect.size() > MyConstant.ZERO) {
            //根据证书级别
            Map<String, List<SysPersonnel>> collectMap = collect.stream().collect(Collectors.groupingBy(SysPersonnel::getPGrade, Collectors.toList()));
            //x坐标
            ArrayList<String> xAxis = Lists.newArrayList();
            //y坐标数据
            ArrayList<Integer> series = Lists.newArrayList();
            HashMap<String, Integer> mapxy = Maps.newHashMap();
            for (String key : collectMap.keySet()) {
                mapxy.put(key, collectMap.get(key).size());
            }
            // 3. 使用Iterator遍历
            Iterator<Map.Entry<String, String>> it = MyConstant.GRADEMAP.entrySet().iterator();
            while (it.hasNext()) {
                Map.Entry<String, String> entry = it.next();
                if (!mapxy.containsKey(entry.getKey())) {
                    mapxy.put(entry.getKey(), null);
                }
            }
            Iterator<Map.Entry<String, Integer>> iterator = mapxy.entrySet().iterator();
            while (iterator.hasNext()) {
                Map.Entry<String, Integer> next = iterator.next();
                xAxis.add(MyConstant.GRADEMAP.get(next.getKey()));
                series.add(next.getValue());
            }
            returnMAP.put("xAxis", xAxis);
            returnMAP.put("series", series);
        }
        return returnMAP;
    }

    @Override
    public SysPersonnel getByWarehouseId(Integer depotId, Integer warehouseId) {
        SysPersonnel personnel = null;
        List<SysPersonnel> byDepotIdAndCfid = personnelMapper.getByDepotIdAndCfid(depotId, warehouseId);
        if (byDepotIdAndCfid.size() > 0) {
            personnel = byDepotIdAndCfid.get(MyConstant.ZERO);
        }

        return personnel;
    }

    @Override
    public ArrayList<HashMap<String, String>> getAll(int depotId) {
        QueryWrapper<SysPersonnel> ew = new QueryWrapper<>();
        ew.eq("depot_id", depotId);
        List<SysPersonnel> personnels = personnelMapper.selectList(ew);
        ArrayList<HashMap<String, String>> objects = Lists.newArrayList();
        if (personnels.size() > 0) {
            for (SysPersonnel r : personnels) {
                HashMap<String, String> map = Maps.newHashMap();
                map.put("personnelId", String.valueOf(r.getPId()));
                map.put("personnelName", r.getUName());
                objects.add(map);
            }
        }
        return objects;
    }

    @Override
    public HashMap<Integer, String> getDepotIdAll(int depotId) {
        HashMap<Integer, String> returnMap = null;
        QueryWrapper<SysPersonnel> ew = new QueryWrapper<>();
        ew.eq("depot_id", depotId);
        List<SysPersonnel> personnels = personnelMapper.selectList(ew);
        if (personnels.size() > MyConstant.ZERO) {
            returnMap = Maps.newHashMap();
            for (SysPersonnel depot : personnels) {
                returnMap.put(depot.getPId(), depot.getUName());
            }
        }
        return returnMap;
    }

    @Transactional
    @Override
    public String uploadFile(MultipartFile file, Integer userId) throws Exception {
        String str = "";
        if (file != null && userId != null) {
            //1.根据userID查询文件名称
            SysPersonnel personnel = personnelMapper.selectById(userId);
            if (personnel == null) {
                return str;
            }
            if (personnel.getPHeadPortrait() != null) {
                //2.如果存在图片先删除图片在上传图片
                minioUtils.removeObject(MyConstant.BUCKET_NAME, personnel.getPHeadPortrait());
            }
            //上传图片返回文件名称
            str = minioUtils.uploadFile(file, MyConstant.BUCKET_NAME);
            personnel.setPId(userId);
            personnel.setPHeadPortrait(str);
            personnelMapper.updateById(personnel);
        }
        return str;
    }

    @Override
    public ArrayList<HashMap<String, String>> getByDepotIdAndRoleId(Integer depotID, Integer roleID) {
        List<SysPersonnel> byDepotIdAndRoleId = personnelMapper.getByDepotIdAndRoleId(depotID, roleID);
        ArrayList<HashMap<String, String>> objects = Lists.newArrayList();
        if (byDepotIdAndRoleId.size() > 0) {
            for (SysPersonnel r : byDepotIdAndRoleId) {
                HashMap<String, String> map = Maps.newHashMap();
                map.put("personnelId", String.valueOf(r.getPId()));
                map.put("personnelName", r.getUName());
                objects.add(map);
            }
        }
        return objects;
    }

    @Override
    public List<SysPersonnelPO> getByDepotId(Integer id) {
        //获取所有部门信息
        List<SysDepartment> departments = departmentService.getAll();
        //根据库点获取人员信息
        QueryWrapper<SysPersonnel> ew = new QueryWrapper<>();
        ew.eq("depot_id", id);
        ew.eq("u_status", MyConstant.ONE);
        List<SysPersonnel> personnels = personnelMapper.selectList(ew);
        //把人员信息放到封装类里面
        List<SysPersonnelPO> personnelPOS = Lists.newArrayList();
        for (SysPersonnel dep : personnels) {
            personnelPOS.add(new SysPersonnelPO(dep.getDepartmentId(), dep.getPId(), dep.getUName(), dep.getPPost(), dep.getPHeadPortrait(), MyConstant.ONE));
        }
        //部门和人员封装到集合
        List<SysPersonnelPO> departmentStaffs = Lists.newArrayList();
        for (SysDepartment dep : departments) {
            SysPersonnelPO personnelPO = new SysPersonnelPO(dep.getDId(), dep.getDName(), MyConstant.ZERO);
            List<SysPersonnelPO> collect = personnelPOS.stream().filter(personnel -> Objects.equals(personnel.getDepartmentStaffId(), dep.getDId())).collect(Collectors.toList());
            personnelPO.setChildren(collect);
            departmentStaffs.add(personnelPO);
        }
        return departmentStaffs;
    }

    /**
     * @param id
     * @param mType  0表示委托，1表示租仓,3表示所有
     * @param method 0代表添加，1代表更新，2代表删除
     */
    @Override
    public void updateByPerId(Integer id, Integer mType, Integer method) {
        SysPersonnel personnel = personnelMapper.selectById(id);
        if (personnel != null) {
            String alternateFieldsOne = personnel.getAlternateFieldsOne();
            if (method == MyConstant.ZERO) {
                //空代表没有权限，0表示委托，1表示租仓
                if (StringUtils.isNotBlank(alternateFieldsOne)) {
                    personnel.setAlternateFieldsOne(MyConstant.THREE_STR);
                } else {
                    personnel.setAlternateFieldsOne(mType.toString());
                }
            } else if (method == MyConstant.ONE) {
                //空代表没有权限，0表示委托，1表示租仓
                if (StringUtils.isNotBlank(alternateFieldsOne)) {
                    if (alternateFieldsOne.equals(MyConstant.THREE_STR)) {
                        personnel.setAlternateFieldsOne(mType.toString());
                    } else {
                        personnel.setAlternateFieldsOne(MyConstant.THREE_STR);
                    }
                } else {
                    personnel.setAlternateFieldsOne(mType.toString());
                }
            } else if (method == MyConstant.TWO) {
                if (alternateFieldsOne != null) {
                    if (alternateFieldsOne.equals(mType.toString())) {
                        personnel.setAlternateFieldsOne("");
                    }
                    if (alternateFieldsOne.equals(MyConstant.THREE_STR)) {
                        if (mType == MyConstant.ZERO) {
                            mType = MyConstant.ONE;
                        } else if (mType == MyConstant.ONE) {
                            mType = MyConstant.ZERO;
                        }
                        personnel.setAlternateFieldsOne(mType.toString());
                    }
                }
            }
            personnelMapper.updateById(personnel);
        }
    }

    @Override
    public void updateByIdType(Integer id, String mType) {
        SysPersonnel personnel = personnelMapper.selectById(id);
        if (personnel != null) {
            if (StringUtils.isNotBlank(mType)) {
                personnel.setAlternateFieldsTwo(mType);
            } else {
                personnel.setAlternateFieldsTwo("");
            }
            personnelMapper.updateById(personnel);
        }
    }
}
