package com.unissoft.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.unissoft.MyConstant;
import com.unissoft.mapper.SysResourceMapper;
import com.unissoft.model.SysResource;
import com.unissoft.model.SysResourcePO;
import com.unissoft.result.PageParam;
import com.unissoft.service.SysResourceService;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 资源（如前台某个菜单） 服务实现类
 * </p>
 *
 * @author wsm
 * @since 2020-11-25
 */
@Service
public class SysResourceServiceI extends ServiceImpl<SysResourceMapper, SysResource> implements SysResourceService {
    @Autowired
    private SysResourceMapper resourceMapper;
    
    @Override
    public Page<SysResource> getPage(PageParam pageParam) {
        int pageIndex = pageParam.getPageIndex();
        int pageSize = pageParam.getPageSize();
        QueryWrapper<SysResource> ew = new QueryWrapper<>();
        if (!StringUtils.isEmpty(pageParam.getCondition())){

            JSONObject object = new JSONObject(pageParam.getCondition());
            String name = object.optString("seName").trim();
            String code = object.optString("seCode");
            String projectId = object.optString("seProjectId");
            String isEnable = object.optString("seIsEnabled");
            if (!StringUtils.isEmpty(name)) {
                ew.like("se_name", name);
            }
            if (!StringUtils.isEmpty(code)) {
                ew.eq("se_code", code);
            }
            if (!StringUtils.isEmpty(projectId)) {
                ew.eq("se_project_id", projectId);
            }
            if (!StringUtils.isEmpty(isEnable)) {
                ew.eq("se_is_enabled", isEnable);
            }
        }
        ew.orderByDesc("se_create_time");
        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        Page<SysResource> pageResult = resourceMapper.selectPage(page, ew);
        return pageResult;
    }

    @Override
    public List<SysResourcePO> getAll() {
        List<SysResource> resourceAll = this.getAllResource();
        List<SysResourcePO> resource = Lists.newArrayList();
        for (SysResource p:resourceAll){
            resource.add(new SysResourcePO(p.getSeId(),p.getSeName(),p.getSeParentId()));
        }
        List<SysResourcePO> menu = createMenu(resource);
        return menu;
    }

    @Override
    public List<SysResource> getAllResource() {
        List<SysResource> resourceAll = resourceMapper.selectList(new QueryWrapper<>());
        return resourceAll;
    }

    private List<SysResourcePO> createMenu(List<SysResourcePO> mp01_menus) {
        ArrayList<SysResourcePO> rootMenu = new ArrayList<>();

        //查询根节点数据
        for (SysResourcePO menu : mp01_menus) {
            if (menu.getParentCode()==0) {
                rootMenu.add(menu);
            }
        }

        //获取根节点下的子节点
        for (SysResourcePO menu : rootMenu) {
            List<SysResourcePO> child = getChild(menu.getResourceCode(), mp01_menus);
            menu.setChildren(child);
        }
        return rootMenu;
    }

    private List<SysResourcePO> getChild(Integer menuid, List<SysResourcePO> allmp01_menus) {
        //存放子菜单的集合
        ArrayList<SysResourcePO> listChild = new ArrayList<>();
        for (SysResourcePO menu : allmp01_menus) {
            if (menu.getParentCode().equals(menuid)) {
                listChild.add(menu);
            }
        }
        //递归：
        for (SysResourcePO menu : listChild) {
            menu.setChildren(getChild(menu.getResourceCode(), allmp01_menus));
        }
        if (listChild.size() == 0) {
            return null;
        }
        return listChild;
    }
}
