package com.unissoft.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.unissoft.MyConstant;
import com.unissoft.mapper.SysResourceRoleMapper;
import com.unissoft.mapper.SysRoleMapper;
import com.unissoft.model.SysResource;
import com.unissoft.model.SysResourceRole;
import com.unissoft.model.SysRole;
import com.unissoft.result.PageParam;
import com.unissoft.service.SysResourceRoleService;
import com.unissoft.service.SysResourceService;
import com.unissoft.service.SysRoleService;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 角色 服务实现类
 * </p>
 *
 * @author wsm
 * @since 2020-11-25
 */
@Service
public class SysRoleServiceI extends ServiceImpl<SysRoleMapper, SysRole> implements SysRoleService {

    @Autowired
    private SysRoleMapper roleMapper;

    @Autowired
    private SysResourceRoleService resourceRoleService;

    @Autowired
    private SysResourceRoleMapper resourceRoleMapper;

    @Autowired
    private SysResourceService resourceService;

    List<Integer> allResoId = Lists.newArrayList();

    @Override
    public List<Integer> getByColeId(Integer roleId) {
        QueryWrapper<SysResourceRole> ew = new QueryWrapper<>();
        ew.eq("role_id", roleId);
        List<SysResourceRole> sysResourceRoles = resourceRoleMapper.selectList(ew);
        List<Integer> resources = null;
        if (sysResourceRoles.size() > 0) {
            resources = sysResourceRoles.stream().map(SysResourceRole::getResourceId).collect(Collectors.toList());
        }
        return resources;
    }

    @Override
    public synchronized List<Integer> getAllResource(Integer id) {
        //获取所有资源
        List<SysResource> allResource = resourceService.getAllResource();
        //获取该角色下的资源
        List<Integer> byColeId = this.getByColeId(id);

        for (Integer i : byColeId) {
            List<Integer> child = getChild(i, allResource);
            if (child != null) {
                allResoId.addAll(child);
            }
        }
        allResoId.addAll(byColeId);
        List<Integer> collect = allResoId.stream().distinct().filter(t -> t != 0).collect(Collectors.toList());
        allResoId = Lists.newArrayList();
        return collect;
    }

    @Override
    public ArrayList<HashMap<String, String>> getAll() {
        QueryWrapper<SysRole> ew = new QueryWrapper<>();
        List<SysRole> sysRoles = roleMapper.selectList(ew);
        ArrayList<HashMap<String, String>> objects = Lists.newArrayList();
        if (sysRoles.size() > 0) {
            for (SysRole r : sysRoles) {
                HashMap<String, String> map = Maps.newHashMap();
                map.put("roleCode", String.valueOf(r.getRId()));
                map.put("roleName", r.getRName());
                objects.add(map);
            }
        }
        return objects;
    }

    @Override
    public Page<SysRole> getPage(PageParam pageParam) {
        int pageIndex = pageParam.getPageIndex();
        int pageSize = pageParam.getPageSize();
        QueryWrapper<SysRole> ew = new QueryWrapper<>();
        if (!StringUtils.isEmpty(pageParam.getCondition())) {
            JSONObject object = new JSONObject(pageParam.getCondition());
            String name = object.optString("rName").trim();
            String code = object.optString("rCode");
            String projectId = object.optString("rProjectId");
            String isEnable = object.optString("rIsEnabled");
            if (!StringUtils.isEmpty(name)) {
                ew.like("r_name", name);
            }
            if (!StringUtils.isEmpty(code)) {
                ew.eq("r_code", code);
            }
            if (!StringUtils.isEmpty(projectId)) {
                ew.eq("r_project_id", projectId);
            }
            if (!StringUtils.isEmpty(isEnable)) {
                ew.eq("r_is_enabled", isEnable);
            }
        }
        ew.orderByDesc("r_create_time");
        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        Page<SysRole> pageResult = roleMapper.selectPage(page, ew);
        return pageResult;
    }

    @Transactional
    @Override
    public void saveRole(SysRole model) {
        Date date = new Date();
        model.setrCreateTime(date);
        //保存角色信息
        roleMapper.insert(model);
        //添加角色下的资源
        addRoleResources(model);
    }

    @Transactional
    @Override
    public void updateRole(SysRole model) {
        Date date = new Date();
        model.setrUpdateTime(date);
        roleMapper.updateById(model);
        //修改角色下的资源
        addRoleResources(model);
    }

    @Transactional
    @Override
    public void deleteById(Integer id) {
        QueryWrapper<SysResourceRole> qw = new QueryWrapper<>();
        qw.eq("role_id", id);
        resourceRoleService.remove(qw);
        roleMapper.deleteById(id);
    }

    private void addRoleResources(SysRole model) {
        //先删除角色下的资源
        QueryWrapper<SysResourceRole> qw = new QueryWrapper<>();
        qw.eq("role_id", model.getRId());
        resourceRoleService.remove(qw);
        if (!StringUtils.isEmpty(model.getResourceCodes())) {
            String[] resources = model.getResourceCodes().split(",");
            ArrayList<SysResourceRole> resourceRoles = Lists.newArrayList();
            for (String resourceId : resources) {
                SysResourceRole sysResourceRole = new SysResourceRole();
                sysResourceRole.setResourceId(Integer.valueOf(resourceId));
                sysResourceRole.setRoleId(model.getRId());
                resourceRoles.add(sysResourceRole);
            }
            //批量添加角色对应的资源
            resourceRoleService.saveBatch(resourceRoles);
        }
    }


    private List<Integer> getChild(Integer menuid, List<SysResource> allmp01_menus) {
        List<Integer> all = Lists.newArrayList();
        //存放子菜单的集合
        for (SysResource menu : allmp01_menus) {
            if (menu.getSeId().equals(menuid)) {
                all.add(menu.getSeParentId());
            }
        }
        //递归：
        for (Integer menu : all) {
            List<Integer> child = getChild(menu, allmp01_menus);
            if (child != null) {
                allResoId.addAll(child);
            }
        }

        if (all.size() == 0) {
            return null;
        }
        return all;
    }
}
