package com.unissoft.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.unissoft.MyConstant;
import com.unissoft.asyncs.ServerRefresh;
import com.unissoft.config.MyConfig;
import com.unissoft.config.MySecurityMetadataSource;
import com.unissoft.dto.UserVO;
import com.unissoft.feign.OauthFeign;
import com.unissoft.mapper.SysUserMapper;
import com.unissoft.model.*;
import com.unissoft.result.PageParam;
import com.unissoft.result.ResultEnum;
import com.unissoft.result.ResultView;
import com.unissoft.service.*;
import com.unissoft.utils.DateUtil;
import com.unissoft.utils.RedisUtil;
import org.apache.commons.lang.StringUtils;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.DigestUtils;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 用户表 服务实现类
 * </p>
 *
 * @author wsm
 * @since 2020-11-25
 */
@Service
public class SysUserServiceI extends ServiceImpl<SysUserMapper, SysUser> implements SysUserService {

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    private RedisUtil redisUtil;

    @Autowired
    private OauthFeign oauthFeign;

    @Autowired
    private MySecurityMetadataSource mySecurityMetadataSource;

    @Autowired
    private ServerRefresh serverRefresh;

    @Autowired
    private SysUserMapper userMapper;

    @Autowired
    private OauthClientDetailsService oauthClientDetailsService;

    @Autowired
    private MyConfig myConfig;

    @Autowired
    private SysPersonnelService personnelService;

    @Autowired
    private SysUserRoleService userRoleService;

    @Autowired
    private BasicsUserWarehouseService userWarehouseService;

    @Override
    public IPage<SysUserPO> getPage(PageParam pageParam) {
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();
        Page<SysUserPO> page = null;
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page<SysUserPO>(pageIndex, pageSize);
        }
        String depotId = null, roleId = null, uName = null, personnelName = null;
        if (StringUtils.isNotBlank(pageParam.getCondition())) {
            JSONObject object = new JSONObject(pageParam.getCondition());
            depotId = object.optString("depotId");
            roleId = object.optString("roleId");
            uName = object.optString("uName").trim();
            personnelName = object.optString("personnelName").trim();

            if (StringUtils.isNotBlank(uName)) {
                uName = "%" + uName + "%";
            }
            if (StringUtils.isNotBlank(personnelName)) {
                personnelName = "%" + personnelName + "%";
            }
        }

        IPage<SysUserPO> page1 = userMapper.getPage(page, depotId, roleId, uName, personnelName);

        return page1;
    }

    @Override
    public SysUserPO getByUserId(Integer id) {
        SysUserPO byUserId = userMapper.getByUserId(id);
        /*if (byUserId != null) {
            Map byUserIdCF = userMapper.getByUserIdCF(id);
            if (byUserIdCF != null && byUserIdCF.get("cfId") != null) {
                byUserId.setStorehouseIds(byUserIdCF.get("cfId").toString());
                byUserId.setStorehouseNames(byUserIdCF.get("cfName").toString());
            }
        }*/
        return byUserId;
    }

    @Override
    public ResultView login(String body) {

        JSONObject jsonObject = new JSONObject(body);
        String name = jsonObject.optString("uName");
        String password = jsonObject.optString("uPassword");
        String typeCode = jsonObject.optString("typeCode");
        //查账号是否存在
        if (StringUtils.isEmpty(name) || StringUtils.isEmpty(password)) {
            return ResultView.error(ResultEnum.CODE_5);
        }
        //判断该用户是否已经登录，已登录退出登录。保证一个用户不能再多台设备登录
        Object userName = redisUtil.get(name);
        if (userName != null) {
            //退出登录
            oauthFeign.logout(userName.toString());
            //删除用户信息
            redisUtil.del(name);
        }
        //查询用户信息
        String pas = DigestUtils.md5DigestAsHex(password.getBytes());
        SysUserPO byUserNameAndPassword = userMapper.getByUserNameAndPassword(name, pas);
        if (byUserNameAndPassword == null) {
            return ResultView.error(ResultEnum.CODE_5);
        }
        //查 OauthClientDetails 信息
        OauthClientDetailsModel oauthClientDetailsModel = oauthClientDetailsService.getById(myConfig.getClientId());
        if (oauthClientDetailsModel == null) {
            return ResultView.error(ResultEnum.CODE_7);
        }
        //获取token,封装登录需要的参数
        MultiValueMap<String, Object> paramMap = encapsulationLogin(oauthClientDetailsModel, name, pas);
        Map map1 = restTemplate.postForObject(MyConstant.OAUTH_TOKEN, paramMap, Map.class);
        Map map = new HashMap();
        map.put("user", byUserNameAndPassword);
        map.put("tokenInfo", map1);
        //登录成功保存用户信息到redis，用于实现一个用户只能在一台设备登录
        redisUtil.set(name, map1.get(MyConstant.ACCESS_TOKEN), MyConstant.TIME_OUT);
        //保存Android设备码
        personnelService.updateByIdType(byUserNameAndPassword.getPersonnelId(), typeCode);
        //保存用户登录记录，后期做统计使用
        Date dayTime = DateUtil.getDayTime();
        List<Object> userVO = Lists.newArrayList(JSON.toJSONString(new UserVO(byUserNameAndPassword.getUId(),dayTime)));
        redisUtil.lSetList(MyConstant.SPECIFIC_TIME, userVO);
        return ResultView.success(map);
    }

    @Override
    public ResultView logout(String token, String personnelId) {
        if (StringUtils.isNotBlank(personnelId)) {
            personnelService.updateByIdType(Integer.valueOf(personnelId), "");
        }
        ResultView resultView = oauthFeign.logout(token);
        return resultView;
    }

    @Override
    public void loadResourceDefine() {
        //刷新本地服务
        mySecurityMetadataSource.loadResourceDefine();
        //调用刷新其它服务
        serverRefresh.serverRefreshs();
    }

    @Override
    public void register(String body) {
        JSONObject jsonObject = new JSONObject(body);
        String name = jsonObject.getString("name");
        String password = jsonObject.getString("password");

        //查账号是否存在
        QueryWrapper<SysUser> qw = new QueryWrapper<>();
        qw.eq("name", name);
        int count = userMapper.selectCount(qw);
        //注册
        if (count > 0) {
            SysUser model = new SysUser();
            model.setuName(name);
            model.setuPassword(password);
            model.setuCreateTime(new Date());
            userMapper.insert(model);
        }
    }

    @Transactional
    @Override
    public void add(SysUser model) {
        Date date = new Date();
        model.setuCreateTime(date);
        if (StringUtils.isNotBlank(model.getuPassword())) {
            //md5 加密
            model.setuPassword(DigestUtils.md5DigestAsHex(model.getuPassword().getBytes()));
        }
        userMapper.insert(model);
        if (StringUtils.isNotBlank(model.getStorehouseIds()) && model.getPersonnelId() != null) {
            //添加人员下有那些仓房信息
            SysPersonnel personnel = new SysPersonnel();
            Integer personnelId = model.getPersonnelId();
            personnel.setPId(personnelId);
            personnel.setStorehouseIds(model.getStorehouseIds());
            personnel.setStorehouseNames(model.getStorehouseNames());
            personnelService.updateById(personnel);
            //解析仓房信息，保存到人员仓房关系表
            this.updateUserWarehouse(model.getStorehouseIds(), model.getuId(), personnelId, model.getRoleIds());
        }
        //关联角色信息
        addUserRole(model);
    }

    @Transactional
    @Override
    public void update(SysUser model) {
        Date date = new Date();
        model.setuUpdateTime(date);
        userMapper.updateById(model);
        //添加人员下有那些仓房信息
        SysPersonnel personnel = new SysPersonnel();
        Integer personnelId = model.getPersonnelId();
        personnel.setPId(personnelId);
        if (StringUtils.isNotBlank(model.getStorehouseIds())) {
            personnel.setStorehouseNames(model.getStorehouseNames());
            personnel.setStorehouseIds(model.getStorehouseIds());
            //解析仓房信息，保存到人员仓房关系表
            this.updateUserWarehouse(model.getStorehouseIds(), model.getuId(), personnelId, model.getRoleIds());
        } else {
            personnel.setStorehouseNames("");
            personnel.setStorehouseIds("");
        }
        personnelService.updateById(personnel);
        //关联角色信息
        addUserRole(model);
    }

    @Override
    public void deleteById(int id, int status) {
        SysUser model = new SysUser();
        model.setuId(id);
        model.setUStatus(status);
        userMapper.updateById(model);
    }

    @Override
    public Integer getByUserName(String userName) {
        QueryWrapper<SysUser> qw = new QueryWrapper<>();
        qw.eq("u_name", userName);
        Integer integer = userMapper.selectCount(qw);
        return integer;
    }

    @Override
    public Integer getIfPassword(int id, String password) {
        QueryWrapper<SysUser> qw = new QueryWrapper<>();
        qw.eq("u_id", id);
        qw.eq("u_password", DigestUtils.md5DigestAsHex(password.getBytes()));
        Integer integer = userMapper.selectCount(qw);
        return integer;
    }

    @Override
    public Map<Integer, String> getNoticeUserInfo(Integer depotId, String recvType, String recvDetails) {
        List<SysUser> sysUsers = null;
        QueryWrapper<SysUser> qw = new QueryWrapper<>();
        qw.eq("u_status", MyConstant.ONE);
        if (recvType.equals(MyConstant.TEN)) {
            //recvType = 10 返回系统所有用户信息 recvType = 10 返回当前用户所在粮库的所有用户
            if (depotId == MyConstant.ONE) {
                //1代表分公司返回所有用户信息
                sysUsers = userMapper.selectList(qw);
            } else {
                //返回当前用户所在粮库的所有用户
                String[] split = depotId.toString().split(",");
                sysUsers = userMapper.getByDepotIds(split);
            }
        } else if (recvType.equals(MyConstant.TWENTY)) {
            //userId 是粮库人员,且如果 recvType = 20 返回null
            if (depotId != MyConstant.ONE) {
                return null;
            } else if (StringUtils.isNotBlank(recvDetails)) {
                //  返回recvDetails中的组织id对应的所有用户,多个粮库id
                String[] split = recvDetails.split(",");
                sysUsers = userMapper.getByDepotIds(split);
            }
        } else if (recvType.equals(MyConstant.THIRTY)) {
            //且如果 recvType = 30 返回当前用户所在分公司的所有用户 且如果 recvType = 30 返回null
            //  30 返回当前用户所在分公司的所有用户
            if (depotId != MyConstant.ONE) {
                return null;
            } else {
                sysUsers = userMapper.selectList(qw);
            }

        } else if (recvType.equals(MyConstant.FORTY) && StringUtils.isNotBlank(recvDetails)) {
            // 且如果 (recvType = 40 且 recvDetails不为空)，返回recvDetails中的角色id对应的所有用户
            String[] split = recvDetails.split(",");
            sysUsers = userMapper.getByRoleIds(depotId, split);
        }
        //遍历结果集返回
        HashMap<Integer, String> returnUsers = getReturnUsers(sysUsers);
        return returnUsers;
    }

    @Override
    public List<SysUser> getUsersByRoleIds(Integer depotId, String roleIds) {
        String[] split = roleIds.split(",");

        List<SysUser> sysUsers = userMapper.getByRoleIdsAndDepotId(split, depotId);

        return sysUsers;
    }

    @Override
    public Map<Integer, String> getAll(Integer depotId) {
        String[] split = null;
        if (depotId != null) {
            split = depotId.toString().split(",");
        }
        List<SysUser> byRoleIds = userMapper.getByDepotIds(split);
        //遍历结果集返回
        HashMap<Integer, String> map = null;
        if (byRoleIds != null && byRoleIds.size() > MyConstant.ZERO) {
            map = Maps.newHashMap();
            for (SysUser user : byRoleIds) {
                map.put(user.getuId(), user.getPersonnelName());
            }
        }
        return map;
    }

    @Override
    public Map<String, List<SysUser>> getRoleIdAll(String roleIds, Integer depotId) {
        Map<String, List<SysUser>> collect = null;
        String[] split = roleIds.split(",");
        List<SysUser> byRoleIdsAndDepotId = userMapper.getByRoleIdsAndDepotId(split, depotId);
        if (byRoleIdsAndDepotId.size() > MyConstant.ZERO) {
            collect = byRoleIdsAndDepotId.stream().collect(Collectors.groupingBy(SysUser::getRoleIds, Collectors.toList()));
        }
        return collect;
    }

    @Override
    public HashMap<Integer, SysUserDTO> getByUserAll() {
        List<SysUserDTO> byUserAll = userMapper.getByUserAll();
        HashMap<Integer, SysUserDTO> map = Maps.newHashMap();
        for (SysUserDTO user:byUserAll) {
            map.put(user.getUId(),user);
        }
        return map;
    }

    private void addUserRole(SysUser model) {
        //先删除角色下的资源
        QueryWrapper<SysUserRole> qw = new QueryWrapper<>();
        qw.eq("user_id", model.getuId());
        userRoleService.remove(qw);
        if (StringUtils.isNotBlank(model.getRoleIds())) {
            String[] roles = model.getRoleIds().split(",");
            ArrayList<SysUserRole> userRoles = Lists.newArrayList();
            for (String roleId : roles) {
                SysUserRole userRole = new SysUserRole();
                userRole.setRoleId(Integer.valueOf(roleId));
                userRole.setUserId(model.getuId());
                userRoles.add(userRole);
            }
            //批量添加用户对应的角色
            userRoleService.saveBatch(userRoles);
        }
    }

    private HashMap<Integer, String> getReturnUsers(List<SysUser> sysUsers) {
        HashMap<Integer, String> map = null;
        //遍历结果集返回
        if (sysUsers != null && sysUsers.size() > MyConstant.ZERO) {
            map = Maps.newHashMap();
            for (SysUser user : sysUsers) {
                map.put(user.getuId(), user.getuName());
            }
        }
        return map;
    }

    private void updateUserWarehouse(String cfIds, Integer userId, Integer personnelId, String role) {
        String[] split = cfIds.split(",");
        ArrayList<BasicsUserWarehouse> userWarehouses = Lists.newArrayList();
        for (String cf : split) {
            BasicsUserWarehouse userWarehouse = new BasicsUserWarehouse();
            userWarehouse.setUserId(userId);
            userWarehouse.setPersonnelId(personnelId);
            userWarehouse.setWarehouseId(Integer.valueOf(cf));
            userWarehouse.setRoleId(Integer.valueOf(role));
            userWarehouses.add(userWarehouse);
            //删除关联信息保管员角色的数据
            if (role.equals(MyConstant.THIRTY)) {
                userWarehouseService.removeCF(Integer.valueOf(cf), null);
            }
        }
        //删除关联信息
        userWarehouseService.removeCF(null, personnelId);
        //批量保存
        userWarehouseService.saveBatch(userWarehouses);
    }

    /**
     * 封装登录需要的参数
     *
     * @param oauthClientDetailsModel
     * @param name                    用户名
     * @param pas                     加密后密码
     * @return
     */
    private MultiValueMap<String, Object> encapsulationLogin(OauthClientDetailsModel oauthClientDetailsModel, String name, String pas) {
        MultiValueMap<String, Object> paramMap = new LinkedMultiValueMap<>();
        paramMap.add(MyConstant.CLIENT_ID, myConfig.getClientId());
        paramMap.add(MyConstant.CLIENT_SECRET, oauthClientDetailsModel.getClientSecret());
        paramMap.add(MyConstant.USERNAME, name);
        paramMap.add(MyConstant.PASSWORD, pas);
        paramMap.add(MyConstant.GRANT_TYPE, MyConstant.PASSWORD);
        paramMap.add(MyConstant.SCOPE, oauthClientDetailsModel.getScope());
        return paramMap;
    }

}
